@@ -4,7 +4,7 @@
 #*                  This file is part of the program and library             *
 #*         SCIP --- Solving Constraint Integer Programs                      *
 #*                                                                           *
-#*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      *
+#*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      *
 #*                                                                           *
 #*  Licensed under the Apache License, Version 2.0 (the "License");          *
 #*  you may not use this file except in compliance with the License.         *
@@ -48,6 +48,7 @@ SOLBASENAME=${17}  # - base name for solution file
 VISUALIZE=${18}    # - true, if the branch-and-bound search should be visualized
 SOLUFILE=${19}     # - solu file, only necessary if ${SETCUTOFF} is 1
 EMPHBENCHMARK="${20}"  # - use set emphasis benchmark
+CLOCKTYPE="${21}"      # - clocktype (1 = CPU, 2 = wallclock)
 
 
 #args=("$@")
@@ -110,7 +111,7 @@ then
 fi
 
 echo "set timelimit ${TIMELIMIT}"            >> "${TMPFILE}"
-echo "set clocktype 0"                       >> "${TMPFILE}"
+echo "set clocktype ${CLOCKTYPE}"            >> "${TMPFILE}"
 echo "set mip display 3"                     >> "${TMPFILE}"
 echo "set mip interval ${DISPFREQ}"          >> "${TMPFILE}"
 echo "set mip tolerances mipgap 0.0"         >> "${TMPFILE}"
