@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -32,19 +32,12 @@
 #ifndef __TSPREADER_H__
 #define __TSPREADER_H__
 
-#include <iostream>
-#include <fstream>
+
 #include <string>
-#include "GomoryHuTree.h"
 #include "objscip/objscip.h"
 #include "scip/def.h"
-
-#ifdef SCIP_WITH_ZLIB
-#include "gzstream.h"
-typedef gzstream::igzstream tspifstream;
-#else
-typedef std::ifstream tspifstream;
-#endif
+#include "GomoryHuTree.h"   // for GRAPH
+#include "scip/pub_fileio.h"
 
 
 namespace tsp
@@ -101,8 +94,8 @@ public:
 private:
 
    /** parses the node list */
-   void getNodesFromFile(
-      tspifstream&       filedata,           /**< filestream containing the data to extract */
+   SCIP_RETCODE getNodesFromFile(
+      SCIP_FILE*         file,               /**< file containing the data to extract */
       double*            x_coords,           /**< double array to be filled with the x-coordinates of the nodes */
       double*            y_coords,           /**< same for y-coordinates */
       GRAPH*             graph               /**< the graph which is to be generated by the nodes */
