@@ -29,6 +29,11 @@
 
 #define TMP
 
+//SV disable warning due to cast of unsigned int* to sparsegraph* in SG_VDE macro
+#ifdef __clang__
+#pragma clang diagnostic ignored "-Wcast-align"
+#endif
+
 /*   #define ONE_WORD_SETS  not sure about this!  See notes.txt.  */
 #include "nausparse.h"
 
