@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -156,6 +156,7 @@ SCIP_RETCODE checkSetupTolerances(
 static
 SCIP_RETCODE resolveNLPWithTighterFeastol(
    SCIP*                 subproblem,         /**< the SCIP data structure */
+   SCIP_BENDERS*         benders,            /**< the benders' decomposition structure */
    SCIP_Real             multiplier,         /**< the amount by which to decrease the tolerance */
    SCIP_Bool*            success             /**< TRUE is the resolving of the LP was successful */
    )
@@ -164,7 +165,7 @@ SCIP_RETCODE resolveNLPWithTighterFeastol(
 #ifdef SCIP_DEBUG
    SCIP_NLPTERMSTAT nlptermstat;
 #endif
-   SCIP_NLPPARAM nlpparam = SCIP_NLPPARAM_DEFAULT(subproblem);  /*lint !e446*/
+   SCIP_NLPPARAM nlpparam = SCIPbendersGetNLPParam(benders);
 #ifdef SCIP_MOREDEBUG
    SCIP_SOL* nlpsol;
 #endif
@@ -759,7 +760,7 @@ SCIP_DECL_BENDERSCUTEXEC(benderscutExecOpt)
 
             while( multiplier > 1e-06 && (*result) == SCIP_DIDNOTFIND )
             {
-               SCIP_CALL( resolveNLPWithTighterFeastol(subproblem, multiplier, &success) );
+               SCIP_CALL( resolveNLPWithTighterFeastol(subproblem, benders, multiplier, &success) );
 
                if( success )
                {
