@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -65,6 +65,9 @@
 #include "scip/scip_sol.h"
 #include "scip/scip_tree.h"
 #include "scip/scip_var.h"
+#include "scip/symmetry_graph.h"
+#include "symmetry/struct_symmetry.h"
+#include <string.h>
 
 
 /* constraint handler properties */
@@ -274,7 +277,7 @@ SCIP_RETCODE consdataCatchEvents(
    assert(consdata != NULL);
 
    /* catch bound change events for both bounds on resultant variable */
-   SCIP_CALL( SCIPcatchVarEvent(scip, consdata->resvar, SCIP_EVENTTYPE_BOUNDCHANGED, 
+   SCIP_CALL( SCIPcatchVarEvent(scip, consdata->resvar, SCIP_EVENTTYPE_BOUNDCHANGED,
          eventhdlr, (SCIP_EVENTDATA*)consdata, NULL) );
 
    /* catch tightening events for lower bound and relaxed events for upper bounds on operator variables */
@@ -699,7 +702,7 @@ SCIP_RETCODE applyFixings(
  *   - for each operator variable vi:  resvar - vi            >= 0
  *   - one additional row:             resvar - v1 - ... - vn <= 0
  */
-static 
+static
 SCIP_RETCODE createRelaxation(
    SCIP*                 scip,               /**< SCIP data structure */
    SCIP_CONS*            cons                /**< constraint to check */
@@ -813,17 +816,17 @@ SCIP_RETCODE checkCons(
          mustcheck = !SCIProwIsInLP(consdata->rows[r]);
          if( mustcheck )
             break;
-      }         
+      }
    }
 
    /* check feasibility of constraint if necessary */
    if( mustcheck )
    {
+      SCIP_Real maxsolval = 0.0;
+      SCIP_Real sumsolval = 0.0;
       SCIP_Real solval;
-      SCIP_Real maxsolval;
-      SCIP_Real sumsolval;
       SCIP_Real viol;
-      int maxsolind;
+      int maxsolind = 0;
       int i;
 
       /* increase age of constraint; age is reset to zero, if a violation was found only in case we are in
@@ -834,16 +837,12 @@ SCIP_RETCODE checkCons(
          SCIP_CALL( SCIPincConsAge(scip, cons) );
       }
 
-      maxsolind = 0;
-      maxsolval = 0.0;
-      sumsolval = 0.0;
-
       /* evaluate operator variables */
       for( i = 0; i < consdata->nvars; ++i )
       {
          solval = SCIPgetSolVal(scip, sol, consdata->vars[i]);
 
-         if( solval > maxsolval )
+         if( maxsolval < solval )
          {
             maxsolind = i;
             maxsolval = solval;
@@ -887,6 +886,7 @@ SCIP_RETCODE checkCons(
          }
       }
 
+      /* update constraint violation in solution */
       if( sol != NULL )
          SCIPupdateSolConsViolation(scip, sol, viol, viol);
    }
@@ -1395,6 +1395,65 @@ SCIP_RETCODE upgradeCons(
    return SCIP_OKAY;
 }
 
+/** adds symmetry information of constraint to a symmetry detection graph */
+static
+SCIP_RETCODE addSymmetryInformation(
+   SCIP*                 scip,               /**< SCIP pointer */
+   SYM_SYMTYPE           symtype,            /**< type of symmetries that need to be added */
+   SCIP_CONS*            cons,               /**< constraint */
+   SYM_GRAPH*            graph,              /**< symmetry detection graph */
+   SCIP_Bool*            success             /**< pointer to store whether symmetry information could be added */
+   )
+{
+   SCIP_CONSDATA* consdata;
+   SCIP_VAR** orvars;
+   SCIP_VAR** vars;
+   SCIP_Real* vals;
+   SCIP_Real constant = 0.0;
+   int nlocvars;
+   int nvars;
+   int i;
+
+   assert(scip != NULL);
+   assert(cons != NULL);
+   assert(graph != NULL);
+   assert(success != NULL);
+
+   consdata = SCIPconsGetData(cons);
+   assert(consdata != NULL);
+
+   /* get active variables of the constraint */
+   nvars = SCIPgetNVars(scip);
+   nlocvars = SCIPgetNVarsOr(scip, cons);
+
+   SCIP_CALL( SCIPallocBufferArray(scip, &vars, nvars) );
+   SCIP_CALL( SCIPallocBufferArray(scip, &vals, nvars) );
+
+   orvars = SCIPgetVarsOr(scip, cons);
+   for( i = 0; i < consdata->nvars; ++i )
+   {
+      vars[i] = orvars[i];
+      vals[i] = 1.0;
+   }
+
+   assert(SCIPgetResultantOr(scip, cons) != NULL);
+   vars[nlocvars] = SCIPgetResultantOr(scip, cons);
+   vals[nlocvars++] = 2.0;
+   assert(nlocvars <= nvars);
+
+   SCIP_CALL( SCIPgetSymActiveVariables(scip, symtype, &vars, &vals, &nlocvars, &constant, SCIPisTransformed(scip)) );
+
+   /* represent the OR constraint via the gadget for linear constraints and use the constant as lhs/rhs to
+    * distinguish different OR constraints (OR constraints do not have an intrinsic right-hand side)
+    */
+   SCIP_CALL( SCIPextendPermsymDetectionGraphLinear(scip, graph, vars, vals, nlocvars,
+         cons, -constant, -constant, success) );
+
+   SCIPfreeBufferArray(scip, &vals);
+   SCIPfreeBufferArray(scip, &vars);
+
+   return SCIP_OKAY;
+}
 
 /*
  * Callback methods of constraint handler
@@ -1467,7 +1526,7 @@ SCIP_DECL_CONSDELETE(consDeleteOr)
 }
 
 
-/** transforms constraint data into data belonging to the transformed problem */ 
+/** transforms constraint data into data belonging to the transformed problem */
 static
 SCIP_DECL_CONSTRANS(consTransOr)
 {  /*lint --e{715}*/
@@ -1489,7 +1548,7 @@ SCIP_DECL_CONSTRANS(consTransOr)
    SCIP_CALL( SCIPcreateCons(scip, targetcons, SCIPconsGetName(sourcecons), conshdlr, targetdata,
          SCIPconsIsInitial(sourcecons), SCIPconsIsSeparated(sourcecons), SCIPconsIsEnforced(sourcecons),
          SCIPconsIsChecked(sourcecons), SCIPconsIsPropagated(sourcecons),
-         SCIPconsIsLocal(sourcecons), SCIPconsIsModifiable(sourcecons), 
+         SCIPconsIsLocal(sourcecons), SCIPconsIsModifiable(sourcecons),
          SCIPconsIsDynamic(sourcecons), SCIPconsIsRemovable(sourcecons), SCIPconsIsStickingAtNode(sourcecons)) );
 
    return SCIP_OKAY;
@@ -1652,22 +1711,18 @@ SCIP_DECL_CONSENFOPS(consEnfopsOr)
    return SCIP_OKAY;
 }
 
-
-/** feasibility check method of constraint handler for integral solutions */
+/** feasibility check method of constraint handler or */
 static
 SCIP_DECL_CONSCHECK(consCheckOr)
 {  /*lint --e{715}*/
+   SCIP_Bool violated;
    int i;
 
    *result = SCIP_FEASIBLE;
 
-   /* method is called only for integral solutions, because the enforcing priority is negative */
-   for( i = 0; i < nconss && (*result == SCIP_FEASIBLE || completely); i++ )
+   for( i = 0; i < nconss && ( *result == SCIP_FEASIBLE || completely ); ++i )
    {
-      SCIP_Bool violated = FALSE;
-
       SCIP_CALL( checkCons(scip, conss[i], sol, checklprows, printreason, &violated) );
-
       if( violated )
          *result = SCIP_INFEASIBLE;
    }
@@ -1675,7 +1730,6 @@ SCIP_DECL_CONSCHECK(consCheckOr)
    return SCIP_OKAY;
 }
 
-
 /** domain propagation method of constraint handler */
 static
 SCIP_DECL_CONSPROP(consPropOr)
@@ -1893,7 +1947,7 @@ SCIP_DECL_CONSCOPY(consCopyOr)
    resvar = NULL;
 
    /* get variables that need to be copied */
-   sourceresvar = SCIPgetResultantOr(sourcescip, sourcecons); 
+   sourceresvar = SCIPgetResultantOr(sourcescip, sourcecons);
    sourcevars = SCIPgetVarsOr(sourcescip, sourcecons);
    nvars = SCIPgetNVarsOr(sourcescip, sourcecons);
 
@@ -1919,7 +1973,7 @@ SCIP_DECL_CONSCOPY(consCopyOr)
       if( *valid )
       {
          assert(resvar != NULL);
-         SCIP_CALL( SCIPcreateConsOr(scip, cons, SCIPconsGetName(sourcecons), resvar, nvars, vars, 
+         SCIP_CALL( SCIPcreateConsOr(scip, cons, SCIPconsGetName(sourcecons), resvar, nvars, vars,
                initial, separate, enforce, check, propagate, local, modifiable, dynamic, removable, stickingatnode) );
       }
    }
@@ -1952,9 +2006,9 @@ SCIP_DECL_CONSPARSE(consParseOr)
    SCIP_CALL( SCIPduplicateBufferArray(scip, &strcopy, str, (int)(strlen(str)+1)));
 
    /* cutoff "or" form the constraint string */
-   token = SCIPstrtok(strcopy, "=", &saveptr ); 
+   token = SCIPstrtok(strcopy, "=", &saveptr );
 
-   /* parse variable name */ 
+   /* parse variable name */
    SCIP_CALL( SCIPparseVarName(scip, token, &resvar, &endptr) );
 
    if( resvar == NULL )
@@ -1964,10 +2018,10 @@ SCIP_DECL_CONSPARSE(consParseOr)
    else
    {
       /* cutoff "or" form the constraint string */
-      (void) SCIPstrtok(NULL, "(", &saveptr ); 
+      (void) SCIPstrtok(NULL, "(", &saveptr );
 
       /* cutoff ")" form the constraint string */
-      token = SCIPstrtok(NULL, ")", &saveptr ); 
+      token = SCIPstrtok(NULL, ")", &saveptr );
 
       varssize = 100;
       nvars = 0;
@@ -1995,7 +2049,7 @@ SCIP_DECL_CONSPARSE(consParseOr)
          assert(varssize >= requiredsize);
 
          /* create and constraint */
-         SCIP_CALL( SCIPcreateConsOr(scip, cons, name, resvar, nvars, vars, 
+         SCIP_CALL( SCIPcreateConsOr(scip, cons, name, resvar, nvars, vars,
                initial, separate, enforce, check, propagate, local, modifiable, dynamic, removable, stickingatnode) );
       }
 
@@ -2047,6 +2101,23 @@ SCIP_DECL_CONSGETNVARS(consGetNVarsOr)
    return SCIP_OKAY;
 }
 
+/** constraint handler method which returns the permutation symmetry detection graph of a constraint */
+static
+SCIP_DECL_CONSGETPERMSYMGRAPH(consGetPermsymGraphOr)
+{  /*lint --e{715}*/
+   SCIP_CALL( addSymmetryInformation(scip, SYM_SYMTYPE_PERM, cons, graph, success) );
+
+   return SCIP_OKAY;
+}
+
+/** constraint handler method which returns the signed permutation symmetry detection graph of a constraint */
+static
+SCIP_DECL_CONSGETSIGNEDPERMSYMGRAPH(consGetSignedPermsymGraphOr)
+{  /*lint --e{715}*/
+   SCIP_CALL( addSymmetryInformation(scip, SYM_SYMTYPE_SIGNPERM, cons, graph, success) );
+
+   return SCIP_OKAY;
+}
 
 /*
  * Callback methods of event handler
@@ -2119,6 +2190,8 @@ SCIP_RETCODE SCIPincludeConshdlrOr(
          CONSHDLR_DELAYSEPA) );
    SCIP_CALL( SCIPsetConshdlrTrans(scip, conshdlr, consTransOr) );
    SCIP_CALL( SCIPsetConshdlrEnforelax(scip, conshdlr, consEnforelaxOr) );
+   SCIP_CALL( SCIPsetConshdlrGetPermsymGraph(scip, conshdlr, consGetPermsymGraphOr) );
+   SCIP_CALL( SCIPsetConshdlrGetSignedPermsymGraph(scip, conshdlr, consGetSignedPermsymGraphOr) );
 
    return SCIP_OKAY;
 }
