@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -46,6 +46,7 @@
 #include "scip/cons_nonlinear.h"
 #include "scip/pub_misc.h"
 #include "scip/nlhdlr_bilinear.h"
+#include "symmetry/struct_symmetry.h"
 
 #define EXPRHDLR_NAME         "prod"
 #define EXPRHDLR_DESC         "product expression"
@@ -78,6 +79,8 @@ struct SCIP_ExprData
 struct SCIP_ExprhdlrData
 {
    SCIP_CONSHDLR*        conshdlr;           /**< nonlinear constraint handler (to compute estimates for > 2-dim products) */
+
+   SCIP_Bool             expandalways;       /**< whether to expand products of a sum and several factors in simplify (SP12b) */
 };
 
 /** node for linked list of expressions */
@@ -112,6 +115,7 @@ SCIP_RETCODE buildSimplifiedProduct(
    SCIP*                 scip,               /**< SCIP data structure */
    SCIP_Real             simplifiedcoef,     /**< simplified product should be simplifiedcoef * PI simplifiedfactors */
    EXPRNODE**            simplifiedfactors,  /**< factors of simplified product */
+   SCIP_Bool             expandalways,       /**< whether to expand products of a sum and several factors in simplify (SP12b) */
    SCIP_Bool             changed,            /**< indicates whether some of the simplified factors was changed */
    SCIP_EXPR**           simplifiedexpr,     /**< buffer to store the simplified expression */
    SCIP_DECL_EXPR_OWNERCREATE((*ownercreate)), /**< function to call to create ownerdata */
@@ -891,6 +895,7 @@ SCIP_RETCODE enforceSP12(
    SCIP*                 scip,               /**< SCIP data structure */
    SCIP_Real             simplifiedcoef,     /**< simplified product should be simplifiedcoef * PI simplifiedfactors */
    EXPRNODE*             finalchildren,      /**< factors of simplified product */
+   SCIP_Bool             expandalways,       /**< whether to expand products of a sum and several factors in simplify (SP12b) */
    SCIP_EXPR**           simplifiedexpr,     /**< buffer to store the simplified expression */
    SCIP_DECL_EXPR_OWNERCREATE((*ownercreate)), /**< function to call to create ownerdata */
    void*                 ownercreatedata     /**< data to pass to ownercreate */
@@ -1014,7 +1019,7 @@ SCIP_RETCODE enforceSP12(
             }
 #endif
 
-            SCIP_CALL( buildSimplifiedProduct(scip, 1.0, &finalfactors, TRUE, &term, ownercreate, ownercreatedata) );
+            SCIP_CALL( buildSimplifiedProduct(scip, 1.0, &finalfactors, expandalways, TRUE, &term, ownercreate, ownercreatedata) );
             assert(finalfactors == NULL);
             assert(term != NULL);
 
@@ -1081,7 +1086,7 @@ SCIP_RETCODE enforceSP12(
          SCIP_CALL( simplifyMultiplyChildren(scip, factors, 2, &termcoef, &finalfactors, &dummy, ownercreate, ownercreatedata) );
          assert(termcoef != 0.0);
 
-         SCIP_CALL( buildSimplifiedProduct(scip, 1.0, &finalfactors, TRUE, &term, ownercreate, ownercreatedata) );
+         SCIP_CALL( buildSimplifiedProduct(scip, 1.0, &finalfactors, expandalways, TRUE, &term, ownercreate, ownercreatedata) );
          assert(finalfactors == NULL);
          assert(term != NULL);
 
@@ -1097,6 +1102,115 @@ SCIP_RETCODE enforceSP12(
    return SCIP_OKAY;
 }
 
+/* expands product of one sum and other expressions
+ * -) (prod factor ... factor (sum c s1 ... sn) factor ... factor )
+ *    - c != 0 --> c * factor is simplified (i.e. factor is not sum!)
+ *    - factor ... factor * si may be not be simplified, so put them in a product list and simplify them from there
+ */
+static
+SCIP_RETCODE enforceSP12b(
+   SCIP*                 scip,               /**< SCIP data structure */
+   SCIP_Real             simplifiedcoef,     /**< simplified product should be simplifiedcoef * prod simplifiedfactors */
+   EXPRNODE*             finalchildren,      /**< factors of simplified product */
+   SCIP_EXPR**           simplifiedexpr,     /**< buffer to store the simplified expression */
+   SCIP_DECL_EXPR_OWNERCREATE((*ownercreate)), /**< function to call to create ownerdata */
+   void*                 ownercreatedata     /**< data to pass to ownercreate */
+   )
+{
+   EXPRNODE* sum_node = NULL;
+   EXPRNODE* n;
+   int nfactors = 0;
+   SCIP_EXPR** factors;
+   int nchildren;
+   SCIP_EXPR* expanded;
+   int j;
+
+   /* check whether there is exactly one sum, calc number of factors */
+   for( n = finalchildren; n != NULL; n = n->next )
+   {
+      if( SCIPisExprSum(scip, n->expr) )
+      {
+         if( sum_node == NULL )
+            sum_node = n;
+         else
+            return SCIP_OKAY;  /* more than one sum */
+      }
+      else
+      {
+         ++nfactors;
+      }
+   }
+   if( sum_node == NULL || nfactors == 0 )  /* no sum or no other factors */
+      return SCIP_OKAY;
+
+   /* collect exprs of all factors other than the sum */
+   SCIP_CALL( SCIPallocBufferArray(scip, &factors, nfactors + 1) );
+   for( n = finalchildren, j = 0; n != NULL; n = n->next )
+      if( n != sum_node )
+         factors[j++] = n->expr;
+
+   /* build new sum expression */
+   nchildren = SCIPexprGetNChildren(sum_node->expr);
+   SCIP_CALL( SCIPcreateExprSum(scip, &expanded, 0, NULL, NULL, 0.0, ownercreate, ownercreatedata) );
+
+   /* handle constant-from-sum * factors */
+   if( SCIPgetConstantExprSum(sum_node->expr) != 0.0 )
+   {
+      if( nfactors == 1 )
+      {
+         SCIP_CALL( SCIPappendExprSumExpr(scip, expanded, factors[0], simplifiedcoef * SCIPgetConstantExprSum(sum_node->expr)) );
+      }
+      else
+      {
+         SCIP_Real termcoef = 1.0;
+         SCIP_Bool dummy;
+         EXPRNODE* finalfactors;
+         SCIP_EXPR* term = NULL;
+
+         SCIP_CALL( simplifyMultiplyChildren(scip, factors, nfactors, &termcoef, &finalfactors, &dummy, ownercreate, ownercreatedata) );
+         assert(termcoef != 0.0);
+
+         SCIP_CALL( buildSimplifiedProduct(scip, 1.0, &finalfactors, TRUE, TRUE, &term, ownercreate, ownercreatedata) );
+         assert(finalfactors == NULL);
+         assert(term != NULL);
+
+         SCIP_CALL( SCIPappendExprSumExpr(scip, expanded, term, termcoef * simplifiedcoef * SCIPgetConstantExprSum(sum_node->expr)) );
+         SCIP_CALL( SCIPreleaseExpr(scip, &term) );
+      }
+   }
+
+   for( j = 0; j < nchildren; ++j )
+   {
+      SCIP_Real coef;
+      SCIP_Real termcoef;
+      SCIP_Bool dummy;
+      EXPRNODE* finalfactors;
+      SCIP_EXPR* term = NULL;
+
+      coef = SCIPgetCoefsExprSum(sum_node->expr)[j];
+      factors[nfactors] = SCIPexprGetChildren(sum_node->expr)[j];
+
+      termcoef = coef;
+      SCIP_CALL( simplifyMultiplyChildren(scip, factors, nfactors + 1, &termcoef, &finalfactors, &dummy, ownercreate, ownercreatedata) );
+      assert(termcoef != 0.0);
+
+      SCIP_CALL( buildSimplifiedProduct(scip, 1.0, &finalfactors, TRUE, TRUE, &term, ownercreate, ownercreatedata) );
+      assert(finalfactors == NULL);
+      assert(term != NULL);
+
+      SCIP_CALL( SCIPappendExprSumExpr(scip, expanded, term, termcoef * simplifiedcoef) );
+      SCIP_CALL( SCIPreleaseExpr(scip, &term) );
+   }
+
+   /* simplify the sum */
+   SCIP_CALL( SCIPcallExprSimplify(scip, expanded, simplifiedexpr, ownercreate, ownercreatedata) );
+   SCIP_CALL( SCIPreleaseExpr(scip, &expanded) );
+
+   SCIPfreeBufferArray(scip, &factors);
+
+   return SCIP_OKAY;
+}
+
 /** builds a simplified product from simplifiedfactors
  *
  * @note this function also releases simplifiedfactors
@@ -1106,6 +1220,7 @@ SCIP_RETCODE buildSimplifiedProduct(
    SCIP*                 scip,               /**< SCIP data structure */
    SCIP_Real             simplifiedcoef,     /**< simplified product should be simplifiedcoef * PI simplifiedfactors */
    EXPRNODE**            simplifiedfactors,  /**< factors of simplified product */
+   SCIP_Bool             expandalways,       /**< whether to expand products of a sum and several factors in simplify (SP12b) */
    SCIP_Bool             changed,            /**< indicates whether some of the simplified factors was changed */
    SCIP_EXPR**           simplifiedexpr,     /**< buffer to store the simplified expression */
    SCIP_DECL_EXPR_OWNERCREATE((*ownercreate)), /**< function to call to create ownerdata */
@@ -1123,10 +1238,17 @@ SCIP_RETCODE buildSimplifiedProduct(
    if( *simplifiedexpr != NULL )
       goto CLEANUP;
 
-   SCIP_CALL( enforceSP12(scip, simplifiedcoef, *simplifiedfactors, simplifiedexpr, ownercreate, ownercreatedata) );
+   SCIP_CALL( enforceSP12(scip, simplifiedcoef, *simplifiedfactors, expandalways, simplifiedexpr, ownercreate, ownercreatedata) );
    if( *simplifiedexpr != NULL )
       goto CLEANUP;
 
+   if( expandalways )
+   {
+      SCIP_CALL( enforceSP12b(scip, simplifiedcoef, *simplifiedfactors, simplifiedexpr, ownercreate, ownercreatedata) );
+      if( *simplifiedexpr != NULL )
+         goto CLEANUP;
+   }
+
    SCIP_CALL( enforceSP10(scip, simplifiedcoef, *simplifiedfactors, simplifiedexpr, ownercreate, ownercreatedata) );
    if( *simplifiedexpr != NULL )
       goto CLEANUP;
@@ -1293,7 +1415,7 @@ SCIP_DECL_EXPRSIMPLIFY(simplifyProduct)
 #endif
 
    /* get simplified product from simplified factors in finalchildren */
-   SCIP_CALL( buildSimplifiedProduct(scip, simplifiedcoef, &finalchildren, changed, simplifiedexpr, ownercreate,
+   SCIP_CALL( buildSimplifiedProduct(scip, simplifiedcoef, &finalchildren, SCIPexprhdlrGetData(SCIPexprGetHdlr(expr))->expandalways, changed, simplifiedexpr, ownercreate,
          ownercreatedata) );
    assert(finalchildren == NULL);
 
@@ -2093,6 +2215,24 @@ SCIP_DECL_EXPRINTEGRALITY(integralityProduct)
    return SCIP_OKAY;
 }
 
+/** expression callback to get information for symmetry detection */
+static
+SCIP_DECL_EXPRGETSYMDATA(getSymDataProduct)
+{  /*lint --e{715}*/
+   assert(scip != NULL);
+   assert(expr != NULL);
+   assert(symdata != NULL);
+
+   SCIP_CALL( SCIPallocBlockMemory(scip, symdata) );
+
+   (*symdata)->nconstants = 1;
+   (*symdata)->ncoefficients = 0;
+   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &(*symdata)->constants, 1) );
+   (*symdata)->constants[0] = SCIPgetCoefExprProduct(expr);
+
+   return SCIP_OKAY;
+}
+
 /** creates the handler for product expressions and includes it into SCIP */
 SCIP_RETCODE SCIPincludeExprhdlrProduct(
    SCIP*                 scip                /**< SCIP data structure */
@@ -2122,6 +2262,11 @@ SCIP_RETCODE SCIPincludeExprhdlrProduct(
    SCIPexprhdlrSetCurvature(exprhdlr, curvatureProduct);
    SCIPexprhdlrSetMonotonicity(exprhdlr, monotonicityProduct);
    SCIPexprhdlrSetIntegrality(exprhdlr, integralityProduct);
+   SCIPexprhdlrSetGetSymdata(exprhdlr, getSymDataProduct);
+
+   SCIP_CALL( SCIPaddBoolParam(scip, "expr/" EXPRHDLR_NAME "/expandalways",
+      "whether to expand products of a sum and several factors in simplify",
+      &exprhdlrdata->expandalways, FALSE, FALSE, NULL, NULL) );
 
    return SCIP_OKAY;
 }
