@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -32,6 +32,7 @@
  */
 
 #include <assert.h>
+#include <string.h>
 
 #include "scip/pub_nlhdlr.h"
 #include "scip/nlhdlr.h"
@@ -40,6 +41,7 @@
 #include "scip/scip_mem.h"
 #include "scip/scip_param.h"
 #include "scip/scip_message.h"
+#include "scip/pub_message.h"
 #include "scip/pub_misc.h"
 
 /**@addtogroup PublicNlhdlrInterfaceMethods
@@ -54,6 +56,7 @@
 #undef SCIPnlhdlrSetInitExit
 #undef SCIPnlhdlrSetProp
 #undef SCIPnlhdlrSetSepa
+#undef SCIPnlhdlrSetSollinearize
 #undef SCIPnlhdlrGetName
 #undef SCIPnlhdlrGetDesc
 #undef SCIPnlhdlrGetDetectPriority
@@ -66,6 +69,7 @@
 #undef SCIPnlhdlrHasExitSepa
 #undef SCIPnlhdlrHasEnfo
 #undef SCIPnlhdlrHasEstimate
+#undef SCIPnlhdlrHasSollinearize
 #endif
 
 /** sets the copy handler callback of a nonlinear handler */
@@ -146,6 +150,18 @@ void SCIPnlhdlrSetSepa(
    nlhdlr->exitsepa = exitsepa;
 }
 
+/** sets the solution linearization callback of a nonlinear handler */
+void SCIPnlhdlrSetSollinearize(
+   SCIP_NLHDLR*          nlhdlr,             /**< nonlinear handler */
+   SCIP_DECL_NLHDLRSOLLINEARIZE((*sollinearize)) /**< solution linearization callback */
+   )
+{
+   assert(nlhdlr != NULL);
+   assert(sollinearize != NULL);
+
+   nlhdlr->sollinearize = sollinearize;
+}
+
 /** gives name of nonlinear handler */
 const char* SCIPnlhdlrGetName(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
@@ -211,6 +227,8 @@ SCIP_Bool SCIPnlhdlrHasIntEval(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
    )
 {
+   assert(nlhdlr != NULL);
+
    return nlhdlr->inteval != NULL;
 }
 
@@ -219,6 +237,8 @@ SCIP_Bool SCIPnlhdlrHasReverseProp(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
    )
 {
+   assert(nlhdlr != NULL);
+
    return nlhdlr->reverseprop != NULL;
 }
 
@@ -227,6 +247,8 @@ SCIP_Bool SCIPnlhdlrHasInitSepa(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
    )
 {
+   assert(nlhdlr != NULL);
+
    return nlhdlr->initsepa != NULL;
 }
 
@@ -235,6 +257,8 @@ SCIP_Bool SCIPnlhdlrHasExitSepa(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
    )
 {
+   assert(nlhdlr != NULL);
+
    return nlhdlr->exitsepa != NULL;
 }
 
@@ -243,6 +267,8 @@ SCIP_Bool SCIPnlhdlrHasEnfo(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
    )
 {
+   assert(nlhdlr != NULL);
+
    return nlhdlr->enfo != NULL;
 }
 
@@ -251,9 +277,21 @@ SCIP_Bool SCIPnlhdlrHasEstimate(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
    )
 {
+   assert(nlhdlr != NULL);
+
    return nlhdlr->estimate != NULL;
 }
 
+/** returns whether nonlinear handler implements the solution linearization callback */
+SCIP_Bool SCIPnlhdlrHasSollinearize(
+   SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
+   )
+{
+   assert(nlhdlr != NULL);
+
+   return nlhdlr->sollinearize != NULL;
+}
+
 /** compares two nonlinear handlers by detection priority
  *
  * if handlers have same detection priority, then compare by name
@@ -606,7 +644,7 @@ SCIP_DECL_NLHDLRENFO(SCIPnlhdlrEnfo)
 
    SCIP_CALL( SCIPstartClock(scip, nlhdlr->enfotime) );
    SCIP_CALL( nlhdlr->enfo(scip, conshdlr, cons, nlhdlr, expr, nlhdlrexprdata, sol, auxvalue,
-         overestimate, allowweakcuts, separated, addbranchscores, result) );
+         overestimate, allowweakcuts, separated, addbranchscores, branchcandonly, result) );
    SCIP_CALL( SCIPstopClock(scip, nlhdlr->enfotime) );
 
    /* update statistics */
@@ -614,6 +652,7 @@ SCIP_DECL_NLHDLRENFO(SCIPnlhdlrEnfo)
    switch( *result )
    {
       case SCIP_SEPARATED :
+         assert(!branchcandonly);
          ++nlhdlr->nseparated;
          break;
       case SCIP_BRANCHED:
@@ -623,6 +662,7 @@ SCIP_DECL_NLHDLRENFO(SCIPnlhdlrEnfo)
          ++nlhdlr->ncutoffs;
          break;
       case SCIP_REDUCEDDOM:
+         assert(!branchcandonly);
          ++nlhdlr->ndomreds;
          break;
       default: ;
@@ -667,6 +707,20 @@ SCIP_DECL_NLHDLRESTIMATE(SCIPnlhdlrEstimate)
    return SCIP_OKAY;
 }
 
+/** call the solution notification callback of a nonlinear handler */
+SCIP_DECL_NLHDLRSOLLINEARIZE(SCIPnlhdlrSollinearize)
+{
+   assert(scip != NULL);
+   assert(nlhdlr != NULL);
+
+   if( nlhdlr->sollinearize == NULL )
+      return SCIP_OKAY;
+
+   SCIP_CALL( nlhdlr->sollinearize(scip, conshdlr, cons, nlhdlr, expr, nlhdlrexprdata, sol, solisbest, overestimate, underestimate) );
+
+   return SCIP_OKAY;
+}
+
 /** reset number of detections counter for last round */
 void SCIPnlhdlrResetNDetectionslast(
    SCIP_NLHDLR*          nlhdlr              /**< nonlinear handler */
