@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -258,7 +258,7 @@ SCIP_RETCODE addSCVarIndicator(
    /* move entries if needed */
    for( i = scvdata->nbnds; i > pos; --i )
    {
-      /* coverity[forward_null] */
+      /* coverity[var_deref_op] */
       scvdata->bvars[i] = scvdata->bvars[i-1];
       scvdata->vals0[i] = scvdata->vals0[i-1];
       scvdata->lbs1[i] = scvdata->lbs1[i-1];
@@ -419,7 +419,7 @@ SCIP_RETCODE varIsSemicontinuous(
    assert(vubvars != NULL || nvubs == 0);
    for( c = 0; c < nvubs; ++c )
    {
-      /* coverity[forward_null] */
+      /* coverity[var_deref_op] */
       if( SCIPvarGetType(vubvars[c]) != SCIP_VARTYPE_BINARY )  /*lint !e613*/
          continue;
 
@@ -1568,6 +1568,13 @@ SCIP_DECL_NLHDLRENFO(nlhdlrEnfoPerspective)
       return SCIP_OKAY;
    }
 
+   if( branchcandonly )
+   {
+      /* let the regular calls to the nlhdlrs after perspective register branching candidates */
+      *result = SCIP_DIDNOTRUN;
+      return SCIP_OKAY;
+   }
+
    auxvar = SCIPgetExprAuxVarNonlinear(expr);
    assert(auxvar != NULL);
 
