@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -28,8 +28,6 @@
  * @author Jane Doe
  */
 
-#include <string.h>
-
 #include "scip/nlhdlr_xyz.h"
 #include "scip/cons_nonlinear.h"
 #include "scip/pub_misc_rowprep.h"
@@ -219,6 +217,21 @@ SCIP_DECL_NLHDLRESTIMATE(nlhdlrEstimateXyz)
 #endif
 
 
+/** nonlinear handler solution linearization callback */
+#if !1
+static
+SCIP_DECL_NLHDLRSOLLINEARIZE(nlhdlrSollinearizeXyz)
+{ /*lint --e{715}*/
+   SCIPerrorMessage("method of xyz nonlinear handler not implemented yet\n");
+   SCIPABORT(); /*lint --e{527}*/
+
+   return SCIP_OKAY;
+}
+#else
+#define nlhdlrSollinearizeXyz NULL
+#endif
+
+
 /** nonlinear handler interval evaluation callback */
 #if 0
 static
@@ -253,7 +266,7 @@ SCIP_DECL_NLHDLRREVERSEPROP(nlhdlrReversepropXyz)
  * nonlinear handler specific interface methods
  */
 
-/** includes Xyz nonlinear handler to consexpr */
+/** includes Xyz nonlinear handler in nonlinear constraint handler */
 SCIP_RETCODE SCIPincludeNlhdlrXyz(
    SCIP*                 scip                /**< SCIP data structure */
    )
@@ -278,6 +291,7 @@ SCIP_RETCODE SCIPincludeNlhdlrXyz(
    SCIPnlhdlrSetInitExit(nlhdlr, nlhdlrInitXyz, nlhdlrExitXyz);
    SCIPnlhdlrSetSepa(nlhdlr, nlhdlrInitSepaXyz, nlhdlrEnfoXyz, nlhdlrEstimateXyz, nlhdlrExitSepaXyz);
    SCIPnlhdlrSetProp(nlhdlr, nlhdlrIntevalXyz, nlhdlrReversepropXyz);
+   SCIPnlhdlrSetSollinearize(nlhdlr, nlhdlrSollinearizeXyz);
 
    return SCIP_OKAY;
 }
