@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -87,23 +87,25 @@ SCIP_RETCODE SCIPincludePresolMILP(
 #include "papilo/core/ProblemBuilder.hpp"
 #include "papilo/Config.hpp"
 
-#define PRESOL_NAME            "milp"
-#define PRESOL_DESC            "MILP specific presolving methods"
-#define PRESOL_PRIORITY        9999999 /**< priority of the presolver (>= 0: before, < 0: after constraint handlers); combined with propagators */
-#define PRESOL_MAXROUNDS             -1 /**< maximal number of presolving rounds the presolver participates in (-1: no limit) */
-#define PRESOL_TIMING           SCIP_PRESOLTIMING_MEDIUM /* timing of the presolver (fast, medium, or exhaustive) */
+#define PRESOL_NAME                "milp"
+#define PRESOL_DESC                "MILP specific presolving methods"
+#define PRESOL_PRIORITY            9999999   /**< priority of the presolver (>= 0: before, < 0: after constraint handlers); combined with propagators */
+#define PRESOL_MAXROUNDS           -1        /**< maximal number of presolving rounds the presolver participates in (-1: no limit) */
+#define PRESOL_TIMING              SCIP_PRESOLTIMING_MEDIUM /* timing of the presolver (fast, medium, or exhaustive) */
 
 /* default parameter values */
 #define DEFAULT_THREADS            1         /**< maximum number of threads presolving may use (0: automatic) */
 #define DEFAULT_MAXFILLINPERSUBST  3         /**< maximal possible fillin for substitutions to be considered */
 #define DEFAULT_MAXSHIFTPERROW     10        /**< maximal amount of nonzeros allowed to be shifted to make space for substitutions */
 #define DEFAULT_DETECTLINDEP       0         /**< should linear dependent equations and free columns be removed? (0: never, 1: for LPs, 2: always) */
-#define DEFAULT_MAXBADGESIZE_SEQ   15000     /**< the max badge size in Probing if PaPILO is executed in sequential mode*/
-#define DEFAULT_MAXBADGESIZE_PAR   -1        /**< the max badge size in Probing if PaPILO is executed in parallel mode*/
+#define DEFAULT_MAXBADGESIZE_SEQ   15000     /**< the max badge size in Probing if PaPILO is executed in sequential mode */
+#define DEFAULT_MAXBADGESIZE_PAR   -1        /**< the max badge size in Probing if PaPILO is executed in parallel mode */
+#define DEFAULT_INTERNAL_MAXROUNDS -1        /**< internal max rounds in PaPILO (-1: no limit, 0: model cleanup) */
 #define DEFAULT_RANDOMSEED         0         /**< the random seed used for randomization of tie breaking */
 #define DEFAULT_MODIFYCONSFAC      0.8       /**< modify SCIP constraints when the number of nonzeros or rows is at most this
                                               *   factor times the number of nonzeros or rows before presolving */
 #define DEFAULT_MARKOWITZTOLERANCE 0.01      /**< the markowitz tolerance used for substitutions */
+#define DEFAULT_VERBOSITY          0
 #define DEFAULT_HUGEBOUND          1e8       /**< absolute bound value that is considered too huge for activitity based calculations */
 #define DEFAULT_ENABLEPARALLELROWS TRUE      /**< should the parallel rows presolver be enabled within the presolve library? */
 #define DEFAULT_ENABLEDOMCOL       TRUE      /**< should the dominated column presolver be enabled within the presolve library? */
@@ -124,11 +126,14 @@ struct SCIP_PresolData
    int lastnrows;                            /**< the number of rows from the last call */
    int threads;                              /**< maximum number of threads presolving may use (0: automatic) */
    int maxfillinpersubstitution;             /**< maximal possible fillin for substitutions to be considered */
-   int maxbadgesizeseq;                      /**< the max badge size in Probing if PaPILO is called in sequential mode*/
+   int maxbadgesizeseq;                      /**< the max badge size in Probing if PaPILO is called in sequential mode */
    int maxbadgesizepar;                      /**< the max badge size in Probing if PaPILO is called in parallel mode */
+   int internalmaxrounds;                    /**< internal max rounds in PaPILO (-1: no limit, 0: model cleanup) */
    int maxshiftperrow;                       /**< maximal amount of nonzeros allowed to be shifted to make space for substitutions */
    int detectlineardependency;               /**< should linear dependent equations and free columns be removed? (0: never, 1: for LPs, 2: always) */
    int randomseed;                           /**< the random seed used for randomization of tie breaking */
+   int verbosity;
+
    SCIP_Bool enablesparsify;                 /**< should the sparsify presolver be enabled within the presolve library? */
    SCIP_Bool enabledomcol;                   /**< should the dominated column presolver be enabled within the presolve library? */
    SCIP_Bool enableprobing;                  /**< should the probing presolver be enabled within the presolve library? */
@@ -296,7 +301,7 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
       return SCIP_OKAY;
    }
 
-   /* only allow communication of constraint modifications by deleting all constraints when they have not been upgraded yet */
+   /* only allow communication of constraint modifications by deleting all constraints when some already have been upgraded */
    SCIP_CONSHDLR* linconshdlr = SCIPfindConshdlr(scip, "linear");
    assert(linconshdlr != NULL);
    bool allowconsmodification = (SCIPconshdlrGetNCheckConss(linconshdlr) == SCIPmatrixGetNRows(matrix));
@@ -318,7 +323,7 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
     * presolve library those look like normal substitution on the postsolve stack */
    presolve.getPresolveOptions().removeslackvars = false;
 
-   /* communicate the SCIP parameters to the presolve libary */
+   /* communicate the SCIP parameters to the presolve library */
    presolve.getPresolveOptions().maxfillinpersubstitution = data->maxfillinpersubstitution;
    presolve.getPresolveOptions().markowitz_tolerance = data->markowitztolerance;
    presolve.getPresolveOptions().maxshiftperrow = data->maxshiftperrow;
@@ -330,8 +335,19 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
    /* communicate the random seed */
    presolve.getPresolveOptions().randomseed = SCIPinitializeRandomSeed(scip, (unsigned int)data->randomseed);
 
+#ifdef PAPILO_TBB
    /* set number of threads to be used for presolve */
    presolve.getPresolveOptions().threads = data->threads;
+#else
+   if (data->threads != DEFAULT_THREADS)
+      SCIPverbMessage(scip, SCIP_VERBLEVEL_HIGH, NULL,
+                      "PaPILO can utilize only multiple threads if it is build with TBB.\n");
+   presolve.getPresolveOptions().threads = 1;
+#endif
+
+#if PAPILO_VERSION_MAJOR > 2 || (PAPILO_VERSION_MAJOR == 2 && PAPILO_VERSION_MINOR >= 3)
+   presolve.getPresolveOptions().maxrounds = data->internalmaxrounds;
+#endif
 
    /* disable dual reductions that are not permitted */
    if( !complete )
@@ -414,7 +430,7 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
 #ifdef SCIP_PRESOLLIB_ENABLE_OUTPUT
    problem.setName(SCIPgetProbName(scip));
 #else
-   presolve.setVerbosityLevel(VerbosityLevel::kQuiet);
+   presolve.setVerbosityLevel((VerbosityLevel) data->verbosity);
 #endif
 
    /* communicate the time limit */
@@ -424,6 +440,8 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
 
    if( 0 != strncmp(data->filename, DEFAULT_FILENAME_PROBLEM, strlen(DEFAULT_FILENAME_PROBLEM)) )
    {
+      SCIPverbMessage(scip, SCIP_VERBLEVEL_HIGH, NULL,
+                      "   writing transformed problem to %s (only enforced constraints)\n", data->filename);
       SCIP_CALL(SCIPwriteTransProblem(scip, data->filename, NULL, FALSE));
    }
 
@@ -475,14 +493,65 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
    }
 
    /* result indicated success, now populate the changes into the SCIP structures */
+
+   /* tighten bounds of variables that are still present after presolving */
+   VariableDomains<SCIP_Real>& varDomains = problem.getVariableDomains();
+   for( int i = 0; i != problem.getNCols(); ++i )
+   {
+      assert( ! varDomains.flags[i].test(ColFlag::kInactive) );
+      SCIP_VAR* var = SCIPmatrixGetVar(matrix, res.postsolve.origcol_mapping[i]);
+      if( !varDomains.flags[i].test(ColFlag::kLbInf) )
+      {
+         SCIP_Bool infeas;
+         SCIP_Bool tightened;
+         SCIP_CALL( SCIPtightenVarLb(scip, var, varDomains.lower_bounds[i], TRUE, &infeas, &tightened) );
+
+         if( tightened )
+            *nchgbds += 1;
+
+         if( infeas )
+         {
+            *result = SCIP_CUTOFF;
+            break;
+         }
+      }
+
+      if( !varDomains.flags[i].test(ColFlag::kUbInf) )
+      {
+         SCIP_Bool infeas;
+         SCIP_Bool tightened;
+         SCIP_CALL( SCIPtightenVarUb(scip, var, varDomains.upper_bounds[i], TRUE, &infeas, &tightened) );
+
+         if( tightened )
+            *nchgbds += 1;
+
+         if( infeas )
+         {
+            *result = SCIP_CUTOFF;
+            break;
+         }
+      }
+   }
+
+   if( *result == SCIP_CUTOFF )
+   {
+      SCIPverbMessage(scip, SCIP_VERBLEVEL_HIGH, NULL,
+            "   (%.1fs) MILP presolver detected infeasibility\n",
+            SCIPgetSolvingTime(scip));
+      SCIPmatrixFree(scip, &matrix);
+      return SCIP_OKAY;
+   }
+
+   /* transfer variable fixings and aggregations */
    std::vector<SCIP_VAR*> tmpvars;
    std::vector<SCIP_Real> tmpvals;
 
-   /* if the number of nonzeros decreased by a sufficient factor, rather create all constraints from scratch */
+   /* if the size of the problem decreased by a sufficient factor, create all constraints from scratch if allowed */
    int newnnz = problem.getConstraintMatrix().getNnz();
    bool constraintsReplaced = false;
    if( newnnz == 0 || (allowconsmodification &&
-         (problem.getNRows() <= data->modifyconsfac * data->lastnrows ||
+         (problem.getNCols() <= data->modifyconsfac * SCIPmatrixGetNColumns(matrix) ||
+          problem.getNRows() <= data->modifyconsfac * SCIPmatrixGetNRows(matrix) ||
           newnnz <= data->modifyconsfac * oldnnz)) )
    {
       int oldnrows = SCIPmatrixGetNRows(matrix);
@@ -540,6 +609,22 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
       }
    }
 
+   /* PaPILO's aggregations are valid regarding the constraints as they were presolved by PaPILO.
+    * If coefficients were changed, but constraints in SCIP are not replaced by those from PaPILO,
+    * then it can not be guaranteed that the bounds of multiaggregated variables will be enforced,
+    * i.e., will be implied by the constraints in SCIP (see also #3704).
+    * Only for variable aggregations, SCIP will ensure this by tightening the bounds on the aggregation
+    * variable as part of SCIPaggregateVars(). For multiaggregations, we will only accept those
+    * where we can be sure with a simple check that the bounds on the aggregated variable are implied.
+    */
+   bool checkmultaggr =
+#if PAPILO_VERSION_MAJOR > 2 || (PAPILO_VERSION_MAJOR == 2 && PAPILO_VERSION_MINOR >= 3)
+         presolve.getStatistics().single_matrix_coefficient_changes > 0
+#else
+         presolve.getStatistics().ncoefchgs > 0
+#endif
+         && !constraintsReplaced;
+
    /* loop over res.postsolve and add all fixed variables and aggregations to scip */
    for( std::size_t i = 0; i != res.postsolve.types.size(); ++i )
    {
@@ -641,8 +726,13 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
          {
             SCIP_Real colCoef = 0.0;
             SCIP_Real updatedSide;
+            SCIP_Bool checklbimplied;
+            SCIP_Bool checkubimplied;
+            SCIP_Real impliedlb;
+            SCIP_Real impliedub;
+            int j;
 
-            for( int j = startRowCoefficients; j < lastRowCoefficients; ++j )
+            for( j = startRowCoefficients; j < lastRowCoefficients; ++j )
             {
                if( res.postsolve.indices[j] == col )
                {
@@ -664,15 +754,76 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
 
             updatedSide = side - constant;
 
-            for( int j = startRowCoefficients; j < lastRowCoefficients; ++j )
+            /* we need to check whether lb/ub on aggrvar is implied by bounds of other variables in multiaggregation
+             * if checkmultaggr is TRUE and the lb/ub is finite
+             * it should be sufficient to ensure global bounds on aggrvar (and as we are in presolve, local=global anyway)
+             */
+            checklbimplied = checkmultaggr && !SCIPisInfinity(scip, -SCIPvarGetLbGlobal(aggrvar));
+            checkubimplied = checkmultaggr && !SCIPisInfinity(scip,  SCIPvarGetUbGlobal(aggrvar));
+            impliedlb = impliedub = updatedSide / colCoef;
+
+            for( j = startRowCoefficients; j < lastRowCoefficients; ++j )
             {
+               SCIP_Real coef;
+               SCIP_VAR* var;
+
                if( res.postsolve.indices[j] == col )
                   continue;
 
-               tmpvars.push_back(SCIPmatrixGetVar(matrix, res.postsolve.indices[j]));
-               tmpvals.push_back(- res.postsolve.values[j] / colCoef);
+               coef = - res.postsolve.values[j] / colCoef;
+               var = SCIPmatrixGetVar(matrix, res.postsolve.indices[j]);
+
+               if( checklbimplied )
+               {
+                  if( coef > 0.0 )
+                  {
+                     /* if impliedlb will be -infinity, then we can give up: we cannot use this mutiaggregation */
+                     if( SCIPisInfinity(scip, -SCIPvarGetLbLocal(var)) )
+                        break;
+                     else
+                        impliedlb += coef * SCIPvarGetLbLocal(var);
+                  }
+                  else
+                  {
+                     if( SCIPisInfinity(scip, SCIPvarGetUbLocal(var)) )
+                        break;
+                     else
+                        impliedlb += coef * SCIPvarGetUbLocal(var);
+                  }
+               }
+
+               if( checkubimplied )
+               {
+                  if( coef > 0.0 )
+                  {
+                     if( SCIPisInfinity(scip, SCIPvarGetUbLocal(var)) )
+                        break;
+                     else
+                        impliedub += coef * SCIPvarGetUbLocal(var);
+                  }
+                  else
+                  {
+                     if( SCIPisInfinity(scip, -SCIPvarGetLbLocal(var)) )
+                        break;
+                     else
+                        impliedub += coef * SCIPvarGetLbLocal(var);
+                  }
+               }
+
+               tmpvals.push_back(coef);
+               tmpvars.push_back(var);
             }
 
+            /* if implied bounds are not sufficient to ensure bounds on aggrvar, then we cannot use the multiaggregation */
+            if( j < lastRowCoefficients )
+               break;
+
+            if( checklbimplied && SCIPisGT(scip, SCIPvarGetLbGlobal(aggrvar), impliedlb) )
+               break;
+
+            if( checkubimplied && SCIPisLT(scip, SCIPvarGetUbGlobal(aggrvar), impliedub) )
+               break;
+
             SCIP_CALL( SCIPmultiaggregateVar(scip, aggrvar, tmpvars.size(),
                tmpvars.data(), tmpvals.data(), updatedSide / colCoef, &infeas, &aggregated) );
          }
@@ -759,48 +910,6 @@ SCIP_DECL_PRESOLEXEC(presolExecMILP)
       }
    }
 
-   /* tighten bounds of variables that are still present after presolving */
-   if( *result != SCIP_CUTOFF )
-   {
-      VariableDomains<SCIP_Real>& varDomains = problem.getVariableDomains();
-      for( int i = 0; i != problem.getNCols(); ++i )
-      {
-         assert( ! varDomains.flags[i].test(ColFlag::kInactive) );
-         SCIP_VAR* var = SCIPmatrixGetVar(matrix, res.postsolve.origcol_mapping[i]);
-         if( !varDomains.flags[i].test(ColFlag::kLbInf) )
-         {
-            SCIP_Bool infeas;
-            SCIP_Bool tightened;
-            SCIP_CALL( SCIPtightenVarLb(scip, var, varDomains.lower_bounds[i], TRUE, &infeas, &tightened) );
-
-            if( tightened )
-               *nchgbds += 1;
-
-            if( infeas )
-            {
-               *result = SCIP_CUTOFF;
-               break;
-            }
-         }
-
-         if( !varDomains.flags[i].test(ColFlag::kUbInf) )
-         {
-            SCIP_Bool infeas;
-            SCIP_Bool tightened;
-            SCIP_CALL( SCIPtightenVarUb(scip, var, varDomains.upper_bounds[i], TRUE, &infeas, &tightened) );
-
-            if( tightened )
-               *nchgbds += 1;
-
-            if( infeas )
-            {
-               *result = SCIP_CUTOFF;
-               break;
-            }
-         }
-      }
-   }
-
    /* finish with a final verb message and return */
    SCIPverbMessage(scip, SCIP_VERBLEVEL_HIGH, NULL,
       "   (%.1fs) MILP presolver (%d rounds): %d aggregations, %d fixings, %d bound changes\n",
@@ -833,10 +942,14 @@ SCIP_RETCODE SCIPincludePresolMILP(
    String name = fmt::format("PaPILO {}.{}.{}", PAPILO_VERSION_MAJOR, PAPILO_VERSION_MINOR, PAPILO_VERSION_PATCH);
 #endif
 
-#ifdef PAPILO_GITHASH_AVAILABLE
-   String desc = fmt::format("parallel presolve for integer and linear optimization (github.com/scipopt/papilo) [GitHash: {}]", PAPILO_GITHASH);
-#else
+#if defined(PAPILO_GITHASH_AVAILABLE) && defined(PAPILO_TBB)
+   String desc = fmt::format("parallel presolve for integer and linear optimization (github.com/scipopt/papilo) (built with TBB) [GitHash: {}]", PAPILO_GITHASH);
+#elif !defined(PAPILO_GITHASH_AVAILABLE) && !defined(PAPILO_TBB)
    String desc("parallel presolve for integer and linear optimization (github.com/scipopt/papilo)");
+#elif defined(PAPILO_GITHASH_AVAILABLE) && !defined(PAPILO_TBB)
+   String desc = fmt::format("parallel presolve for integer and linear optimization (github.com/scipopt/papilo) [GitHash: {}]", PAPILO_GITHASH);
+#elif !defined(PAPILO_GITHASH_AVAILABLE) && defined(PAPILO_TBB)
+   String desc = fmt::format("parallel presolve for integer and linear optimization (github.com/scipopt/papilo) (built with TBB)");
 #endif
 
    /* add external code info for the presolve library */
@@ -921,6 +1034,14 @@ SCIP_RETCODE SCIPincludePresolMILP(
    presoldata->maxbadgesizepar = DEFAULT_MAXBADGESIZE_PAR;
 #endif
 
+#if PAPILO_VERSION_MAJOR > 2 || (PAPILO_VERSION_MAJOR == 2 && PAPILO_VERSION_MINOR >= 3)
+   SCIP_CALL(SCIPaddIntParam(scip, "presolving/" PRESOL_NAME "/internalmaxrounds",
+         "internal maxrounds for each milp presolving (-1: no limit, 0: model cleanup)",
+         &presoldata->internalmaxrounds, TRUE, DEFAULT_INTERNAL_MAXROUNDS, -1, INT_MAX, NULL, NULL));
+#else
+   presoldata->internalmaxrounds = DEFAULT_INTERNAL_MAXROUNDS;
+#endif
+
    SCIP_CALL( SCIPaddBoolParam(scip,
          "presolving/" PRESOL_NAME "/enableparallelrows",
          "should the parallel rows presolver be enabled within the presolve library?",
@@ -952,9 +1073,13 @@ SCIP_RETCODE SCIPincludePresolMILP(
          &presoldata->enablesparsify, TRUE, DEFAULT_ENABLESPARSIFY, NULL, NULL) );
 
    SCIP_CALL( SCIPaddStringParam(scip, "presolving/" PRESOL_NAME "/probfilename",
-         "filename to store the problem before MILP presolving starts",
+         "filename to store the problem before MILP presolving starts (only enforced constraints)",
          &presoldata->filename, TRUE, DEFAULT_FILENAME_PROBLEM, NULL, NULL) );
 
+   SCIP_CALL(SCIPaddIntParam(scip, "presolving/" PRESOL_NAME "/verbosity",
+         "verbosity level of PaPILO (0: quiet, 1: errors, 2: warnings, 3: normal, 4: detailed)",
+         &presoldata->verbosity, FALSE, DEFAULT_VERBOSITY, 0, 4, NULL, NULL));
+
    return SCIP_OKAY;
 }
 
