@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -154,6 +154,11 @@ SCIP_RETCODE probEnsureConssMem(
 
       newsize = SCIPsetCalcMemGrowSize(set, num);
       SCIP_ALLOC( BMSreallocMemoryArray(&prob->conss, newsize) );
+      /* resize sorted original constraints if they exist */
+      if( prob->origcheckconss != NULL )
+      {
+         SCIP_ALLOC( BMSreallocMemoryArray(&prob->origcheckconss, newsize) );
+      }
       prob->consssize = newsize;
    }
    assert(num <= prob->consssize);
@@ -209,6 +214,7 @@ SCIP_RETCODE SCIPprobCopy(
                                               *   target variables */
    SCIP_HASHMAP*         consmap,            /**< a hashmap to store the mapping of source constraints to the corresponding
                                               *   target constraints */
+   SCIP_Bool             original,           /**< copy original or transformed problem? */
    SCIP_Bool             global              /**< create a global or a local copy? */
    )
 {
@@ -229,7 +235,7 @@ SCIP_RETCODE SCIPprobCopy(
    /* call user copy callback method */
    if( sourceprob->probdata != NULL && sourceprob->probcopy != NULL )
    {
-      SCIP_CALL( sourceprob->probcopy(set->scip, sourcescip, sourceprob->probdata, varmap, consmap, &targetdata, global, &result) );
+      SCIP_CALL( sourceprob->probcopy(set->scip, sourcescip, sourceprob->probdata, varmap, consmap, &targetdata, original, global, &result) );
 
       /* evaluate result */
       if( result != SCIP_DIDNOTRUN && result != SCIP_SUCCESS )
@@ -320,6 +326,7 @@ SCIP_RETCODE SCIPprobCreate(
    else
       (*prob)->consnames = NULL;
    (*prob)->conss = NULL;
+   (*prob)->origcheckconss = NULL;
    (*prob)->consssize = 0;
    (*prob)->nconss = 0;
    (*prob)->maxnconss = 0;
@@ -334,6 +341,7 @@ SCIP_RETCODE SCIPprobCreate(
    (*prob)->transformed = transformed;
    (*prob)->nlpenabled = FALSE;
    (*prob)->permuted = FALSE;
+   (*prob)->consschecksorted = FALSE;
    (*prob)->conscompression = FALSE;
 
    return SCIP_OKAY;
@@ -448,6 +456,7 @@ SCIP_RETCODE SCIPprobFree(
    }
 
    /* free constraint array */
+   BMSfreeMemoryArrayNull(&(*prob)->origcheckconss);
    BMSfreeMemoryArrayNull(&(*prob)->conss);
 
    /* free user problem data */
@@ -702,6 +711,30 @@ void SCIPprobResortVars(
    }
 }
 
+/** possibly create and sort the constraints according to check priorties */
+SCIP_RETCODE SCIPprobSortConssCheck(
+   SCIP_PROB*            prob                /**< problem data */
+   )
+{
+   if( prob->consschecksorted || prob->transformed )
+      return SCIP_OKAY;
+
+   if( prob->nconss > 0 )
+   {
+      /* possibly create and copy constraints */
+      if( prob->origcheckconss == NULL )
+      {
+         SCIP_ALLOC( BMSduplicateMemoryArray(&prob->origcheckconss, prob->conss, prob->consssize) );
+      }
+      assert( prob->origcheckconss != NULL );
+
+      /* sort original constraint according to check priority */
+      SCIPsortPtr((void**)prob->origcheckconss, SCIPconsCompCheck, prob->nconss);
+   }
+   prob->consschecksorted = TRUE;
+
+   return SCIP_OKAY;
+}
 
 
 /*
@@ -1313,8 +1346,11 @@ SCIP_RETCODE SCIPprobAddCons(
    /* add the constraint to the problem's constraint array */
    SCIP_CALL( probEnsureConssMem(prob, set, prob->nconss+1) );
    prob->conss[prob->nconss] = cons;
+   if( prob->origcheckconss != NULL )
+      prob->origcheckconss[prob->nconss] = cons;
    prob->nconss++;
    prob->maxnconss = MAX(prob->maxnconss, prob->nconss);
+   prob->consschecksorted = FALSE;
    stat->nactiveconssadded++;
 
    /* undelete constraint, if it was globally deleted in the past */
@@ -1397,6 +1433,11 @@ SCIP_RETCODE SCIPprobDelCons(
    assert(prob->conss[arraypos]->addconssetchg == NULL);
    prob->conss[arraypos]->addarraypos = arraypos;
    prob->nconss--;
+   prob->consschecksorted = FALSE;
+
+   /* if we delete constraints then delete array origcheckconss to be sure */
+   if( prob->origcheckconss != NULL )
+      BMSfreeMemoryArray(&prob->origcheckconss);
 
    /* mark the constraint to be no longer in the problem */
    cons->addarraypos = -1;
