@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -42,6 +42,9 @@
 #include "scip/cons_nonlinear.h"
 #include "scip/cons_sos1.h"
 #include "scip/cons_sos2.h"
+#include "scip/cons_and.h"
+#include "scip/cons_or.h"
+#include "scip/cons_xor.h"
 #include "scip/expr_var.h"
 #include "scip/expr_value.h"
 #include "scip/expr_sum.h"
@@ -139,6 +142,9 @@ private:
    // alternatively, one could encapsulate SCIP_EXPR* into a small class that handles proper reference counting
    std::vector<SCIP_EXPR*> exprstorelease;
 
+   // count on variables or constraints added for logical expressions
+   int logiccount;
+
    // SOS constraints
    // collected while handling suffixes in SuffixHandler
    // sosvars maps the SOS index (can be negative) to the indices of the variables in the SOS
@@ -190,6 +196,48 @@ private:
       return false;
    }
 
+   /// returns variable or value for given expression
+   ///
+   /// if expression is variable, ensure that it is a binary variable and set var
+   /// if expression is value, then set val to whether value is nonzero and set var to NULL
+   /// otherwise throw UnsupportedError exception
+   void LogicalExprToVarVal(
+      LogicalExpr        expr,
+      SCIP_VAR*&         var,
+      SCIP_Bool&         val
+      )
+   {
+      assert(expr != NULL);
+
+      if( SCIPisExprVar(scip, expr) )
+      {
+         var = SCIPgetVarExprVar(expr);
+         if( SCIPvarGetType(var) != SCIP_VARTYPE_BINARY )
+         {
+            SCIP_Bool infeas;
+            SCIP_Bool tightened;
+            SCIP_CALL_THROW( SCIPchgVarType(scip, var, SCIP_VARTYPE_BINARY, &infeas) );
+            assert(!infeas);
+            SCIP_CALL_THROW( SCIPtightenVarLbGlobal(scip, var, 0.0, TRUE, &infeas, &tightened) );
+            assert(!infeas);
+            SCIP_CALL_THROW( SCIPtightenVarUbGlobal(scip, var, 1.0, TRUE, &infeas, &tightened) );
+            assert(!infeas);
+         }
+         val = FALSE;  // for scan-build
+
+         return;
+      }
+
+      if( SCIPisExprValue(scip, expr) )
+      {
+         var = NULL;
+         val = SCIPgetValueExprValue(expr) != 0.0;
+         return;
+      }
+
+      OnUnhandled("logical expression must be binary or constant");
+   }
+
 public:
    /// constructor
    ///
@@ -201,6 +249,7 @@ public:
    : scip(scip_),
      probdata(NULL),
      objexpr(NULL),
+     logiccount(0),
      initsol(NULL),
      colfile(NULL),
      rowfile(NULL)
@@ -368,7 +417,7 @@ public:
          }
       }
 
-      // alloc some space for constraints
+      // alloc some space for algebraic constraints
       probdata->nconss = h.num_algebraic_cons;
       SCIP_CALL_THROW( SCIPallocBlockMemoryArray(scip, &probdata->conss, probdata->nconss) );
       nlconslin.resize(h.num_nl_cons);
@@ -395,7 +444,7 @@ public:
          SCIP_CALL_THROW( SCIPcreateConsBasicLinear(scip, &probdata->conss[i], name, 0, NULL, NULL, -SCIPinfinity(scip), SCIPinfinity(scip)) );
       }
 
-      if( h.num_nl_cons == 0 && h.num_integer_vars() == 0 )
+      if( h.num_nl_cons == 0 && h.num_logical_cons == 0 && h.num_integer_vars() == 0 )
          probdata->islp = true;
 
       // alloc space for common expressions
@@ -673,6 +722,21 @@ public:
       }
    }
 
+   /// receives notification of a logical constraint expression
+   void OnLogicalCon(
+      int                index,
+      LogicalExpr        expr
+      )
+   {
+      if( expr != NULL )
+      {
+         SCIP_CONS* cons;
+         SCIP_CALL_THROW( SCIPcreateConsBasicNonlinear(scip, &cons, "logiccons", expr, 1.0, 1.0) );
+         SCIP_CALL_THROW( SCIPaddCons(scip, cons) );
+         SCIP_CALL_THROW( SCIPreleaseCons(scip, &cons) );
+      }
+   }
+
    /// handles linear part of a common expression
    /// sets up a sum expression, if the linear part isn't empty
    class LinearExprHandler
@@ -932,6 +996,10 @@ public:
                }
                break;
 
+            case mp::suf::Kind::CON_BIT:
+               SCIPverbMessage(amplph.scip, SCIP_VERBLEVEL_HIGH, NULL, "Unknown constraint bit suffix <%.*s>. Ignoring.\n", (int)name.size(), name.data());
+               break;
+
             case mp::suf::Kind::VAR:
             {
                if( strncmp(name.data(), "initial", name.size()) == 0 )
@@ -959,13 +1027,25 @@ public:
                }
                break;
 
+            case mp::suf::Kind::VAR_BIT:
+               SCIPverbMessage(amplph.scip, SCIP_VERBLEVEL_HIGH, NULL, "Unknown variable bit suffix <%.*s>. Ignoring.\n", (int)name.size(), name.data());
+               break;
+
             case mp::suf::Kind::OBJ:
                SCIPverbMessage(amplph.scip, SCIP_VERBLEVEL_HIGH, NULL, "Unknown objective suffix <%.*s>. Ignoring.\n", (int)name.size(), name.data());
                break;
 
+            case mp::suf::Kind::OBJ_BIT:
+               SCIPverbMessage(amplph.scip, SCIP_VERBLEVEL_HIGH, NULL, "Unknown objective bit suffix <%.*s>. Ignoring.\n", (int)name.size(), name.data());
+               break;
+
             case mp::suf::Kind::PROBLEM:
                SCIPverbMessage(amplph.scip, SCIP_VERBLEVEL_HIGH, NULL, "Unknown problem suffix <%.*s>. Ignoring.\n", (int)name.size(), name.data());
                break;
+
+            case mp::suf::Kind::PROB_BIT:
+               SCIPverbMessage(amplph.scip, SCIP_VERBLEVEL_HIGH, NULL, "Unknown problem bit suffix <%.*s>. Ignoring.\n", (int)name.size(), name.data());
+               break;
             }
          }
       }
@@ -1138,6 +1218,337 @@ public:
       return LinearConHandler(*this, constraintIndex);
    }
 
+   /// receives notification of a `Boolean value <mp::expr::BOOL>`
+   LogicalExpr OnBool(
+      bool               value
+      )
+   {
+      SCIP_EXPR* expr;
+
+      SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, value ? 1.0 : 0.0, NULL, NULL) );
+
+      // remember that we have to release this expr
+      exprstorelease.push_back(expr);
+
+      return expr;
+   }
+
+   /// receives notification of a `logical not <mp::expr::NOT>`
+   LogicalExpr OnNot(
+      LogicalExpr        arg
+      )
+   {
+      SCIP_EXPR* expr;
+      SCIP_VAR* var;
+      SCIP_Bool val;
+
+      LogicalExprToVarVal(arg, var, val);
+      if( var != NULL )
+      {
+         SCIP_CALL_THROW( SCIPgetNegatedVar(scip, var, &var) );
+         SCIP_CALL_THROW( SCIPcreateExprVar(scip, &expr, var, NULL, NULL) );
+      }
+      else
+      {
+         SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, val ? 1.0 : 0.0, NULL, NULL) );
+      }
+
+      // remember that we have to release this expr
+      exprstorelease.push_back(expr);
+
+      return expr;
+   }
+
+   /// receives notification of a `binary logical expression <mp::expr::FIRST_BINARY_LOGICAL>`
+   LogicalExpr OnBinaryLogical(
+      mp::expr::Kind     kind,
+      LogicalExpr        lhs,
+      LogicalExpr        rhs
+      )
+   {
+      SCIP_VAR* lhsvar = NULL;
+      SCIP_VAR* rhsvar = NULL;
+      SCIP_Bool lhsval;
+      SCIP_Bool rhsval;
+      SCIP_EXPR* expr;
+
+      assert(lhs != NULL);
+      assert(rhs != NULL);
+
+      LogicalExprToVarVal(lhs, lhsvar, lhsval);
+      LogicalExprToVarVal(rhs, rhsvar, rhsval);
+
+      switch( kind )
+      {
+         case mp::expr::OR:
+         {
+            if( lhsvar == NULL && rhsvar == NULL )
+            {
+               SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, lhsval != 0.0 || rhsval != 0.0 ? 1.0 : 0.0, NULL, NULL) );
+               exprstorelease.push_back(expr);
+               break;
+            }
+
+            if( (lhsvar == NULL && lhsval != 0.0) || (rhsvar == NULL && rhsval != 0.0) )
+            {
+               /* nonzero or rhs == 1, lhs or nonzero == 1 */
+               SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, 1.0, NULL, NULL) );
+               exprstorelease.push_back(expr);
+               break;
+            }
+
+            if( lhsvar == NULL )
+            {
+               /* zero or rhs == rhs */
+               assert(lhsval == 0.0);
+               expr = rhs;
+               break;
+            }
+
+            if( rhsvar == NULL )
+            {
+               /* lhs or zero == lhs */
+               assert(rhsval == 0.0);
+               expr = lhs;
+               break;
+            }
+
+            /* create new resvar and constraint resvar = lhsvar or rhsvar */
+            SCIP_VAR* vars[2];
+            SCIP_VAR* resvar;
+            SCIP_CONS* cons;
+
+            std::string name = std::string("_logic") + std::to_string((long long)logiccount++);
+            SCIP_CALL_THROW( SCIPcreateVarBasic(scip, &resvar, name.c_str(), 0.0, 1.0, 0.0, SCIP_VARTYPE_BINARY) );
+            SCIP_CALL_THROW( SCIPaddVar(scip, resvar) );
+            SCIP_CALL_THROW( SCIPcreateExprVar(scip, &expr, resvar, NULL, NULL) );
+            exprstorelease.push_back(expr);
+
+            vars[0] = lhsvar;
+            vars[1] = rhsvar;
+            name += "def";
+            SCIP_CALL_THROW( SCIPcreateConsBasicOr(scip, &cons, name.c_str(), resvar, 2, vars) );
+            SCIP_CALL_THROW( SCIPaddCons(scip, cons) );
+
+            SCIP_CALL_THROW( SCIPreleaseVar(scip, &resvar) );
+            SCIP_CALL_THROW( SCIPreleaseCons(scip, &cons) );
+
+            break;
+         }
+
+         case mp::expr::AND:
+         {
+            if( lhsvar == NULL && rhsvar == NULL )
+            {
+               SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, lhsval != 0.0 && rhsval != 0.0 ? 1.0 : 0.0, NULL, NULL) );
+               exprstorelease.push_back(expr);
+               break;
+            }
+
+            if( (lhsvar == NULL && lhsval == 0.0) || (rhsvar == NULL && rhsval == 0.0) )
+            {
+               /* zero and rhs == 0, lhs and zero == 0 */
+               SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, 0.0, NULL, NULL) );
+               exprstorelease.push_back(expr);
+               break;
+            }
+
+            if( lhsvar == NULL )
+            {
+               /* nonzero and rhs == rhs */
+               assert(lhsval != 0.0);
+               expr = rhs;
+               break;
+            }
+
+            if( rhsvar == NULL )
+            {
+               /* lhs and nonzero == lhs */
+               assert(rhsval != 0.0);
+               expr = lhs;
+               break;
+            }
+
+            /* create new resvar and constraint resvar = lhsvar and rhsvar */
+            SCIP_VAR* vars[2];
+            SCIP_VAR* resvar;
+            SCIP_CONS* cons;
+
+            std::string name = std::string("_logic") + std::to_string((long long)logiccount++);
+            SCIP_CALL_THROW( SCIPcreateVarBasic(scip, &resvar, name.c_str(), 0.0, 1.0, 0.0, SCIP_VARTYPE_BINARY) );
+            SCIP_CALL_THROW( SCIPaddVar(scip, resvar) );
+            SCIP_CALL_THROW( SCIPcreateExprVar(scip, &expr, resvar, NULL, NULL) );
+            exprstorelease.push_back(expr);
+
+            vars[0] = lhsvar;
+            vars[1] = rhsvar;
+            name += "def";
+            SCIP_CALL_THROW( SCIPcreateConsBasicAnd(scip, &cons, name.c_str(), resvar, 2, vars) );
+            SCIP_CALL_THROW( SCIPaddCons(scip, cons) );
+
+            SCIP_CALL_THROW( SCIPreleaseVar(scip, &resvar) );
+            SCIP_CALL_THROW( SCIPreleaseCons(scip, &cons) );
+
+            break;
+         }
+
+         case mp::expr::IFF:
+         {
+            // the IFF operator returns 1 if both operands are nonzero or both are zero and returns zero otherwise
+            // so this is lhs == rhs
+            if( lhsvar == NULL && rhsvar == NULL )
+            {
+               SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, lhsval == rhsval ? 1.0 : 0.0, NULL, NULL) );
+               exprstorelease.push_back(expr);
+               break;
+            }
+
+            if( lhsvar == NULL )
+            {
+               std::swap(lhs, rhs);
+               std::swap(lhsval, rhsval);
+               std::swap(lhsvar, rhsvar);
+            }
+            assert(lhsvar != NULL);
+
+            if( rhsvar == NULL )
+            {
+               // expression is lhsvar == true
+               // so we return lhsvar or ~lhsvar
+               if( rhsval == TRUE )
+               {
+                  expr = lhs;
+               }
+               else
+               {
+                  SCIP_CALL_THROW( SCIPgetNegatedVar(scip, lhsvar, &lhsvar) );
+                  SCIP_CALL_THROW( SCIPcreateExprVar(scip, &expr, lhsvar, NULL, NULL) );
+                  exprstorelease.push_back(expr);
+               }
+               break;
+            }
+
+            // expressions is lhsvar == rhsvar
+            // we create a new variable auxvar and add a constraint xor(auxvar, lhsvar, rhsvar, TRUE)
+            // to ensure auxvar = (lhsvar == rhsvar)
+            SCIP_VAR* vars[3];
+            SCIP_CONS* cons;
+            std::string name = std::string("_logic") + std::to_string((long long)logiccount++);
+            SCIP_CALL_THROW( SCIPcreateVarBasic(scip, &vars[0], name.c_str(), 0.0, 1.0, 0.0, SCIP_VARTYPE_BINARY) );
+            SCIP_CALL_THROW( SCIPaddVar(scip, vars[0]) );
+            SCIP_CALL_THROW( SCIPcreateExprVar(scip, &expr, vars[0], NULL, NULL) );
+            exprstorelease.push_back(expr);
+
+            vars[1] = lhsvar;
+            vars[2] = rhsvar;
+            name += "def";
+            SCIP_CALL_THROW( SCIPcreateConsBasicXor(scip, &cons, name.c_str(), TRUE, 3, vars) );
+            SCIP_CALL_THROW( SCIPaddCons(scip, cons) );
+
+            SCIP_CALL_THROW( SCIPreleaseVar(scip, &vars[0]) );
+            SCIP_CALL_THROW( SCIPreleaseCons(scip, &cons) );
+
+            break;
+         }
+
+         default:
+            OnUnhandled(mp::expr::str(kind));
+            return NULL;
+      }
+
+      return expr;
+   }
+
+   /// receives notification of a `relational expression <mp::expr::FIRST_RELATIONAL>`
+   /// we only handle equality or inequality between binary variables and boolean values here
+   LogicalExpr OnRelational(
+      mp::expr::Kind     kind,
+      NumericExpr        lhs,
+      NumericExpr        rhs
+      )
+   {
+      SCIP_VAR* lhsvar = NULL;
+      SCIP_VAR* rhsvar = NULL;
+      SCIP_Bool lhsval;
+      SCIP_Bool rhsval;
+      SCIP_EXPR* expr;
+
+      assert(lhs != NULL);
+      assert(rhs != NULL);
+
+      LogicalExprToVarVal(lhs, lhsvar, lhsval);
+      LogicalExprToVarVal(rhs, rhsvar, rhsval);
+
+      switch( kind )
+      {
+         case mp::expr::EQ:
+         case mp::expr::NE:
+         {
+            bool isne = (kind == mp::expr::NE);
+            if( lhsvar == NULL && rhsvar == NULL )
+            {
+               SCIP_CALL_THROW( SCIPcreateExprValue(scip, &expr, lhsval == rhsval ? (isne ? 0.0 : 1.0) : (isne ? 1.0 : 0.0), NULL, NULL) );
+               exprstorelease.push_back(expr);
+               break;
+            }
+
+            if( lhsvar == NULL )
+            {
+               std::swap(lhs, rhs);
+               std::swap(lhsval, rhsval);
+               std::swap(lhsvar, rhsvar);
+            }
+            assert(lhsvar != NULL);
+
+            if( rhsvar == NULL )
+            {
+               // expression is lhsvar == true or lhsvar == false if EQ
+               // so we return lhsvar or ~lhsvar, opposite if NE
+               if( rhsval == (isne ? FALSE : TRUE) )
+               {
+                  expr = lhs;
+               }
+               else
+               {
+                  SCIP_CALL_THROW( SCIPgetNegatedVar(scip, lhsvar, &lhsvar) );
+                  SCIP_CALL_THROW( SCIPcreateExprVar(scip, &expr, lhsvar, NULL, NULL) );
+                  exprstorelease.push_back(expr);
+               }
+               break;
+            }
+
+            // expressions is lhsvar == rhsvar or lhsvar != rhsvar
+            // we create a new variable auxvar and add a constraint xor(auxvar, lhsvar, rhsvar, isne ? FALSE : TRUE)
+            // to ensure auxvar = (lhsvar == rhsvar)  or  auxvar = (lhsvar != rhsvar)
+
+            SCIP_VAR* vars[3];
+            SCIP_CONS* cons;
+            std::string name = std::string("_logic") + std::to_string((long long)logiccount++);
+            SCIP_CALL_THROW( SCIPcreateVarBasic(scip, &vars[0], name.c_str(), 0.0, 1.0, 0.0, SCIP_VARTYPE_BINARY) );
+            SCIP_CALL_THROW( SCIPaddVar(scip, vars[0]) );
+            SCIP_CALL_THROW( SCIPcreateExprVar(scip, &expr, vars[0], NULL, NULL) );
+            exprstorelease.push_back(expr);
+
+            vars[1] = lhsvar;
+            vars[2] = rhsvar;
+            name += "def";
+            SCIP_CALL_THROW( SCIPcreateConsBasicXor(scip, &cons, name.c_str(), isne ? FALSE : TRUE, 3, vars) );
+            SCIP_CALL_THROW( SCIPaddCons(scip, cons) );
+
+            SCIP_CALL_THROW( SCIPreleaseVar(scip, &vars[0]) );
+            SCIP_CALL_THROW( SCIPreleaseCons(scip, &cons) );
+
+            break;
+         }
+
+         default:
+            OnUnhandled(mp::expr::str(kind));
+            return NULL;
+      }
+
+      return expr;
+   }
+
    /// receive notification of the end of the input
    ///
    /// - setup all nonlinear constraints and add them to SCIP
@@ -1162,6 +1573,20 @@ public:
          SCIP_CALL_THROW( SCIPaddLinearVarNonlinear(scip, objcons, objvar, -1.0) );
          SCIP_CALL_THROW( SCIPaddCons(scip, objcons) );
 
+         if( initsol != NULL )
+         {
+            /* compute value for objvar in initial solution from other variable values */
+            SCIP_CALL_THROW( SCIPevalExpr(scip, objexpr, initsol, 0) );
+            if( SCIPexprGetEvalValue(objexpr) != SCIP_INVALID )
+            {
+               SCIPsetSolVal(scip, initsol, objvar, SCIPexprGetEvalValue(objexpr));
+            }
+            else
+            {
+               SCIPwarningMessage(scip, "Objective function could not be evaluated in initial point. Domain error.");
+            }
+         }
+
          SCIP_CALL_THROW( SCIPreleaseCons(scip, &objcons) );
          SCIP_CALL_THROW( SCIPreleaseVar(scip, &objvar) );
       }
@@ -1406,7 +1831,7 @@ SCIP_RETCODE SCIPincludeReaderNl(
    SCIP_CALL( SCIPsetReaderCopy(scip, reader, readerCopyNl) );
    SCIP_CALL( SCIPsetReaderRead(scip, reader, readerReadNl) );
 
-   SCIP_CALL( SCIPincludeExternalCodeInformation(scip, "AMPL/MP 4e2d45c4", "AMPL .nl file reader library (github.com/ampl/mp)") );
+   SCIP_CALL( SCIPincludeExternalCodeInformation(scip, "AMPL/MP 690e9e7", "AMPL .nl file reader library (github.com/ampl/mp)") );
 
    return SCIP_OKAY;
 }
@@ -1487,65 +1912,86 @@ SCIP_RETCODE SCIPwriteSolutionNl(
       for( int i = 0; i < probdata->nvars; ++i )
          SCIPinfoMessage(scip, solfile, "%.17g\n", SCIPgetSolVal(scip, SCIPgetBestSol(scip), probdata->vars[i]));
 
-   /* AMPL solve status codes are at http://www.ampl.com/NEW/statuses.html
-    *     number   string       interpretation
-    *    0 -  99   solved       optimal solution found
-    *  100 - 199   solved?      optimal solution indicated, but error likely
-    *  200 - 299   infeasible   constraints cannot be satisfied
-    *  300 - 399   unbounded    objective can be improved without limit
-    *  400 - 499   limit        stopped by a limit that you set (such as on iterations)
-    *  500 - 599   failure      stopped by an error condition in the solver routines
+   /* AMPL solve status codes are at https://mp.ampl.com/details.html#_CPPv4N2mp3sol6StatusE
+    * (mp::sol::Status enum in amplmp/include/mp/common.h)
     */
-   int solve_result_num;
+   int solve_result_num = mp::sol::FAILURE;
    switch( SCIPgetStatus(scip) )
    {
       case SCIP_STATUS_UNKNOWN:
-         solve_result_num = 500;
          break;
       case SCIP_STATUS_USERINTERRUPT:
-         solve_result_num = 450;
+      case SCIP_STATUS_TERMINATE:
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS_INTERRUPT;
+         else
+            solve_result_num = mp::sol::LIMIT_NO_FEAS_INTERRUPT;
          break;
       case SCIP_STATUS_NODELIMIT:
-         solve_result_num = 400;
-         break;
       case SCIP_STATUS_TOTALNODELIMIT:
-         solve_result_num = 401;
-         break;
       case SCIP_STATUS_STALLNODELIMIT:
-         solve_result_num = 402;
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS_NODES;
+         else
+            solve_result_num = mp::sol::LIMIT_NO_FEAS_NODES;
          break;
       case SCIP_STATUS_TIMELIMIT:
-         solve_result_num = 403;
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS_TIME;
+         else
+            solve_result_num = mp::sol::LIMIT_NO_FEAS_TIME;
          break;
       case SCIP_STATUS_MEMLIMIT:
-         solve_result_num = 404;
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS_SOFTMEM;
+         else
+            solve_result_num = mp::sol::LIMIT_NO_FEAS_SOFTMEM;
          break;
       case SCIP_STATUS_GAPLIMIT:
-         solve_result_num = 405;
+         /* there is no enum value for gaplimit, so use "work limit" */
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS_WORK;
+         else
+            solve_result_num = mp::sol::LIMIT_NO_FEAS_WORK;
+         break;
+      case SCIP_STATUS_PRIMALLIMIT:
+         solve_result_num = mp::sol::LIMIT_FEAS_BESTOBJ;
+         break;
+      case SCIP_STATUS_DUALLIMIT:
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS_BESTBND;
+         else
+            solve_result_num = mp::sol::LIMIT_NO_FEAS_BESTBND;
          break;
       case SCIP_STATUS_SOLLIMIT:
-         solve_result_num = 406;
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS_NUMSOLS;
+         else  /* reach solution limit without solution? */
+            solve_result_num = mp::sol::LIMIT_NO_FEAS;
          break;
       case SCIP_STATUS_BESTSOLLIMIT:
-         solve_result_num = 407;
+      case SCIP_STATUS_RESTARTLIMIT:
+         /* rare SCIP specific limits that don't map to an AMPL status */
+         if( haveprimal )
+            solve_result_num = mp::sol::LIMIT_FEAS;
+         else
+            solve_result_num = mp::sol::LIMIT_NO_FEAS;
          break;
       case SCIP_STATUS_OPTIMAL:
-         solve_result_num = 0;
+         solve_result_num = mp::sol::SOLVED;
          break;
       case SCIP_STATUS_INFEASIBLE:
-         solve_result_num = 200;
+         solve_result_num = mp::sol::INFEASIBLE;
          break;
       case SCIP_STATUS_UNBOUNDED:
-         solve_result_num = 300;
+         if( haveprimal )
+            solve_result_num = mp::sol::UNBOUNDED_FEAS;
+         else
+            solve_result_num = mp::sol::UNBOUNDED_NO_FEAS;
          break;
       case SCIP_STATUS_INFORUNBD:
-         solve_result_num = 299;
+         solve_result_num = mp::sol::LIMIT_INF_UNB;
          break;
-      default:
-         /* solve_result_num = 500; */
-         SCIPerrorMessage("invalid status code <%d>\n", SCIPgetStatus(scip));
-         (void) fclose(solfile);
-         return SCIP_INVALIDDATA;
    }
    SCIPinfoMessage(scip, solfile, "objno 0 %d\n", solve_result_num);
 
