@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -1141,8 +1141,8 @@ SCIP_RETCODE readLinearCoefs(
          assert(nlinvars[row] == 0);
 
          nterms = start[row+1] - start[row];
-         SCIP_CALL( SCIPallocBufferArray(scip, &linvars[row], nterms) );
-         SCIP_CALL( SCIPallocBufferArray(scip, &lincoefs[row], nterms) );
+         SCIP_CALL( SCIPallocBlockMemoryArray(scip, &linvars[row], nterms) );
+         SCIP_CALL( SCIPallocBlockMemoryArray(scip, &lincoefs[row], nterms) );
 
          for( pos = start[row]; pos < start[row+1]; ++pos )
          {
@@ -1205,8 +1205,8 @@ SCIP_RETCODE readLinearCoefs(
 
             for( c = 0; c < nconss; ++c )
             {
-               SCIP_CALL( SCIPallocBufferArray(scip, &linvars[c], nlinvars[c]) );
-               SCIP_CALL( SCIPallocBufferArray(scip, &lincoefs[c], nlinvars[c]) );
+               SCIP_CALL( SCIPallocBlockMemoryArray(scip, &linvars[c], nlinvars[c]) );
+               SCIP_CALL( SCIPallocBlockMemoryArray(scip, &lincoefs[c], nlinvars[c]) );
 
                /* reset nlinvars[c] so it can be used for iteration k=1 */
                nlinvars[c] = 0;
@@ -1235,13 +1235,13 @@ SCIP_RETCODE readQuadraticCoefs(
    SCIP_VAR***           quadvars2,          /**< array to store for each constraint the second variables of the quadratic terms */
    SCIP_Real**           quadcoefs,          /**< array to store for each constraint the coefficients of the quadratic terms */
    int*                  nquadterms,         /**< array to store for each constraint the total number of quadratic terms */
+   int*                  termssize,          /**< pointer to store the size of quadvars1, quadvars2, and quadcoefs */
    SCIP_Bool*            doingfine           /**< buffer to indicate whether no errors occurred */
    )
 {
    const XML_NODE* quadcoef;
    const XML_NODE* qterm;
    const char* attrval;
-   int* termssize;
    SCIP_Real coef;
    int nqterms;
    int count;
@@ -1285,8 +1285,6 @@ SCIP_RETCODE readQuadraticCoefs(
 
    assert(vars != NULL);
 
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &termssize, nconss + 1) );
-
    count = 0;
    for( qterm = xmlFirstChild(quadcoef); qterm != NULL; qterm = xmlNextSibl(qterm), ++count )
    {
@@ -1295,13 +1293,13 @@ SCIP_RETCODE readQuadraticCoefs(
       {
          SCIPerrorMessage("Expected <qTerm> node under <quadraticCoefficients> node, but got <%s>\n", xmlGetName(qterm));
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
       if( count >= nqterms )
       {
          SCIPerrorMessage("Too many quadratic terms under <quadraticCoefficients> node, expected %d many, but got at least %d.\n", nqterms, count + 1);
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
 
       /* get constraint index, or -1 for objective */
@@ -1310,7 +1308,7 @@ SCIP_RETCODE readQuadraticCoefs(
       {
          SCIPerrorMessage("Missing \"idx\" attribute in %d'th <qTerm> node under <quadraticCoefficients> node.\n", count);
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
 
       considx = (int)strtol(attrval, (char**)&attrval, 10);
@@ -1318,7 +1316,7 @@ SCIP_RETCODE readQuadraticCoefs(
       {
          SCIPerrorMessage("Invalid value '%s' in \"idx\" attribute of %d'th <qTerm> node under <quadraticCoefficients> node.\n", xmlGetAttrval(qterm, "idx"), count);
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
 
       /* get index of first variable */
@@ -1327,7 +1325,7 @@ SCIP_RETCODE readQuadraticCoefs(
       {
          SCIPerrorMessage("Missing \"idxOne\" attribute in %d'th <qTerm> node under <quadraticCoefficients> node.\n", count);
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
 
       varidx1 = (int)strtol(attrval, (char**)&attrval, 10);
@@ -1335,7 +1333,7 @@ SCIP_RETCODE readQuadraticCoefs(
       {
          SCIPerrorMessage("Invalid value '%s' in \"idxOne\" attribute of %d'th <qTerm> node under <quadraticCoefficients> node.\n", xmlGetAttrval(qterm, "idxOne"), count);
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
 
       /* get index of second variable */
@@ -1344,7 +1342,7 @@ SCIP_RETCODE readQuadraticCoefs(
       {
          SCIPerrorMessage("Missing \"idxTwo\" attribute in %d'th <qTerm> node under <quadraticCoefficients> node.\n", count);
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
 
       varidx2 = (int)strtol(attrval, (char**)&attrval, 10);
@@ -1352,7 +1350,7 @@ SCIP_RETCODE readQuadraticCoefs(
       {
          SCIPerrorMessage("Invalid value '%s' in \"idxTwo\" attribute of %d'th <qTerm> node under <quadraticCoefficients> node.\n", xmlGetAttrval(qterm, "idxTwo"), count);
          *doingfine = FALSE;
-         goto TERMINATE;
+         return SCIP_OKAY;
       }
 
       /* get (optional) coefficient of quadratic term */
@@ -1364,7 +1362,7 @@ SCIP_RETCODE readQuadraticCoefs(
          {
             SCIPerrorMessage("Invalid value '%s' in \"coef\" attribute of %d'th <qTerm> node under <quadraticCoefficients> node.\n", xmlGetAttrval(qterm, "coef"), count);
             *doingfine = FALSE;
-            goto TERMINATE;
+            return SCIP_OKAY;
          }
       }
       else
@@ -1383,10 +1381,13 @@ SCIP_RETCODE readQuadraticCoefs(
 
       if( nquadterms[considx] + 1 > termssize[considx] )
       {
-         termssize[considx] = SCIPcalcMemGrowSize(scip, nquadterms[considx] + 1);
-         SCIP_CALL( SCIPreallocBufferArray(scip, &quadvars1[considx], termssize[considx]) );  /*lint !e866*/
-         SCIP_CALL( SCIPreallocBufferArray(scip, &quadvars2[considx], termssize[considx]) );  /*lint !e866*/
-         SCIP_CALL( SCIPreallocBufferArray(scip, &quadcoefs[considx], termssize[considx]) );  /*lint !e866*/
+         int newsize;
+
+         newsize = SCIPcalcMemGrowSize(scip, nquadterms[considx] + 1);
+         SCIP_CALL( SCIPreallocBlockMemoryArray(scip, &quadvars1[considx], termssize[considx], newsize) );  /*lint !e866*/
+         SCIP_CALL( SCIPreallocBlockMemoryArray(scip, &quadvars2[considx], termssize[considx], newsize) );  /*lint !e866*/
+         SCIP_CALL( SCIPreallocBlockMemoryArray(scip, &quadcoefs[considx], termssize[considx], newsize) );  /*lint !e866*/
+         termssize[considx] = newsize;
       }
 
       quadvars1[considx][nquadterms[considx]] = vars[varidx1];
@@ -1399,12 +1400,8 @@ SCIP_RETCODE readQuadraticCoefs(
    {
       SCIPerrorMessage("Got only %d quadratic terms under <quadraticCoefficients> node, but expected %d many.\n", count, nqterms);
       *doingfine = FALSE;
-      goto TERMINATE;
    }
 
- TERMINATE:
-   SCIPfreeBufferArray(scip, &termssize);
-
    return SCIP_OKAY;
 }
 
@@ -2345,6 +2342,7 @@ SCIP_DECL_READERREAD(readerReadOsil)
    SCIP_VAR*** quadvars2 = NULL;
    SCIP_Real** quadcoefs = NULL;
    int* nquadterms = NULL;
+   int* termssize = NULL;
 
    /* nonlinear parts */
    SCIP_EXPR** nlexprs = NULL;
@@ -2424,15 +2422,16 @@ SCIP_DECL_READERREAD(readerReadOsil)
    if( !doingfine )
       goto CLEANUP;
 
-   /* allocate memory to store constraint information */
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &linvars, nconss + 1) );
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &lincoefs, nconss + 1) );
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &nlinvars, nconss + 1) );
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &quadvars1, nconss + 1) );
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &quadvars2, nconss + 1) );
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &quadcoefs, nconss + 1) );
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &nquadterms, nconss + 1) );
-   SCIP_CALL( SCIPallocClearBufferArray(scip, &nlexprs, nconss + 1) );
+   /* allocate memory to store constraint information (use block memory, since the size have to be reallocaed) */
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &linvars, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &lincoefs, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &nlinvars, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &quadvars1, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &quadvars2, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &quadcoefs, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &nquadterms, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &nlexprs, nconss + 1) );
+   SCIP_CALL( SCIPallocClearBlockMemoryArray(scip, &termssize, nconss + 1) );
 
    /* read linear coefficients matrix */
    SCIP_CALL_TERMINATE( retcode, readLinearCoefs(scip, data, vars, nvars, nconss, linvars, lincoefs, nlinvars, &doingfine), CLEANUP );
@@ -2441,7 +2440,7 @@ SCIP_DECL_READERREAD(readerReadOsil)
 
    /* read quadratic coefficients */
    SCIP_CALL_TERMINATE( retcode, readQuadraticCoefs(scip, data, vars, nvars, nconss, quadvars1, quadvars2, quadcoefs,
-      nquadterms, &doingfine), CLEANUP );
+         nquadterms, termssize, &doingfine), CLEANUP );
    if( !doingfine )
       goto CLEANUP;
 
@@ -2490,22 +2489,23 @@ SCIP_DECL_READERREAD(readerReadOsil)
       }
 
       /* free quadratic parts */
-      SCIPfreeBufferArrayNull(scip, &quadcoefs[c]);
-      SCIPfreeBufferArrayNull(scip, &quadvars1[c]);
-      SCIPfreeBufferArrayNull(scip, &quadvars2[c]);
+      SCIPfreeBlockMemoryArrayNull(scip, &quadcoefs[c], termssize[c]);
+      SCIPfreeBlockMemoryArrayNull(scip, &quadvars2[c], termssize[c]);
+      SCIPfreeBlockMemoryArrayNull(scip, &quadvars1[c], termssize[c]);
 
       /* free linear parts */
-      SCIPfreeBufferArrayNull(scip, &lincoefs[c]);
-      SCIPfreeBufferArrayNull(scip, &linvars[c]);
+      SCIPfreeBlockMemoryArrayNull(scip, &lincoefs[c], nlinvars[c]);
+      SCIPfreeBlockMemoryArrayNull(scip, &linvars[c], nlinvars[c]);
    }
-   SCIPfreeBufferArrayNull(scip, &nlexprs);
-   SCIPfreeBufferArrayNull(scip, &nquadterms);
-   SCIPfreeBufferArrayNull(scip, &quadcoefs);
-   SCIPfreeBufferArrayNull(scip, &quadvars2);
-   SCIPfreeBufferArrayNull(scip, &quadvars1);
-   SCIPfreeBufferArrayNull(scip, &nlinvars);
-   SCIPfreeBufferArrayNull(scip, &lincoefs);
-   SCIPfreeBufferArrayNull(scip, &linvars);
+   SCIPfreeBlockMemoryArrayNull(scip, &termssize, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &nlexprs, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &nquadterms, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &quadcoefs, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &quadvars2, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &quadvars1, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &nlinvars, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &lincoefs, nconss + 1);
+   SCIPfreeBlockMemoryArrayNull(scip, &linvars, nconss + 1);
 
    /* free variables */
    for( i = 0; i < nvars; ++i )
