@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -37,7 +37,7 @@
 
 #include "scip/sepa_minor.h"
 #include "scip/cons_nonlinear.h"
-#include "scip/nlpi_ipopt.h"
+#include "scip/lapack_calls.h"
 
 #define SEPA_NAME              "minor"
 #define SEPA_DESC              "separator to ensure that 2x2 principal minors of X - xx' are positive semi-definite"
@@ -537,7 +537,7 @@ SCIP_RETCODE getEigenValues(
    eigenvecs[8] = yy;
 
    /* use LAPACK to compute the eigenvalues and eigenvectors */
-   if( SCIPcallLapackDsyevIpopt(TRUE, 3, eigenvecs, eigenvals) != SCIP_OKAY )
+   if( SCIPlapackComputeEigenvalues(SCIPbuffer(scip), TRUE, 3, eigenvecs, eigenvals) != SCIP_OKAY )
    {
       SCIPdebugMsg(scip, "Failed to compute eigenvalues and eigenvectors of augmented quadratic form matrix.\n");
       *success = FALSE;
@@ -815,7 +815,7 @@ SCIP_DECL_SEPAEXECLP(sepaExeclpMinor)
    int ncalls;
 
    /* need routine to compute eigenvalues/eigenvectors */
-   if( !SCIPisIpoptAvailableIpopt() )
+   if( ! SCIPlapackIsAvailable() )
       return SCIP_OKAY;
 
    sepadata = SCIPsepaGetData(sepa);
@@ -848,7 +848,7 @@ SCIP_DECL_SEPAEXECSOL(sepaExecsolMinor)
    int ncalls;
 
    /* need routine to compute eigenvalues/eigenvectors */
-   if( !SCIPisIpoptAvailableIpopt() )
+   if( ! SCIPlapackIsAvailable() )
       return SCIP_OKAY;
 
    sepadata = SCIPsepaGetData(sepa);
