@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -236,7 +236,7 @@ void checkData(
 
 static
 void checkCut(
-   SCIP_ROW*             cut,
+   SCIP_ROWPREP*         cut,
    SCIP_VAR**            vars,
    SCIP_Real*            vals,
    SCIP_Real             rhs,
@@ -249,9 +249,12 @@ void checkCut(
    cr_assert_not_null(vars);
    cr_assert_not_null(vals);
 
-   cr_expect_eq(SCIProwGetNNonz(cut), nvars, "expected %d vars in cut, but got %d\n",
-      nvars, SCIProwGetNNonz(cut));
-   cr_expect(SCIPisEQ(scip, SCIProwGetRhs(cut), rhs), "expected rhs = %f, but got %f\n", rhs, SCIProwGetRhs(cut));
+   SCIPmergeRowprepTerms(scip, cut);
+
+   cr_expect_eq(SCIProwprepGetNVars(cut), nvars, "expected %d vars in cut, but got %d\n",
+      nvars, SCIProwprepGetNVars(cut));
+   cr_expect_eq(SCIProwprepGetSidetype(cut), SCIP_SIDETYPE_RIGHT);
+   cr_expect(SCIPisEQ(scip, SCIProwprepGetSide(cut), rhs), "expected rhs = %f, but got %f\n", rhs, SCIProwprepGetSide(cut));
 
    /* FIXME the remaining tests assume a certain order of terms and thus are not invariant to
     * valid permutations
@@ -260,10 +263,10 @@ void checkCut(
 
    for( i = 0; i < nvars; ++i )
    {
-      cr_expect_eq(SCIPcolGetVar(SCIProwGetCols(cut)[i]), vars[i], "expected var%d = %s, but got %s\n",
-         i + 1, SCIPvarGetName(vars[i]), SCIPvarGetName(SCIPcolGetVar(SCIProwGetCols(cut)[i])));
-      cr_expect_eq(SCIProwGetVals(cut)[i], vals[i], "expected val%d = %f, but got %f\n", i + 1, vals[i],
-         SCIProwGetVals(cut)[i]);
+      cr_expect_eq(SCIProwprepGetVars(cut)[i], vars[i], "expected var%d = %s, but got %s\n",
+         i + 1, SCIPvarGetName(vars[i]), SCIPvarGetName(SCIProwprepGetVars(cut)[i]));
+      cr_expect_eq(SCIProwprepGetCoefs(cut)[i], vals[i], "expected val%d = %f, but got %f\n", i + 1, vals[i],
+         SCIProwprepGetCoefs(cut)[i]);
    }
 }
 
@@ -362,7 +365,7 @@ Test(nlhdlrsoc, detectandfree2, .description = "detects simple norm expression")
    SCIP_CALL( SCIPreleaseCons(scip, &cons) );
 }
 
-/* detects SQRT(8 +  2*(x + 1)^2 + 4*(sin(y) - 2)^2 ) as soc expression */
+/* detects sqrt(8 +  2*(x + 1)^2 + 4*(sin(y) - 2)^2 ) as soc expression */
 Test(nlhdlrsoc, detectandfree3, .description = "detects more complex norm expression")
 {
    SCIP_CONS* cons;
@@ -396,9 +399,9 @@ Test(nlhdlrsoc, detectandfree3, .description = "detects more complex norm expres
    /* setup expected data */
    SCIP_EXPR* vars[3] = {xexpr, SCIPexprGetChildren(SCIPexprGetChildren(normexpr)[0])[2], normexpr};
    int nterms = 4;
-   SCIP_Real offsets[4] = {SQRT(2.0), -4.0, SQRT(8.0) /* constant */, 0.0};
+   SCIP_Real offsets[4] = {sqrt(2.0), -4.0, sqrt(8.0) /* constant */, 0.0};
    int nnonzeroes[4] = {1, 1, 0, 1};
-   SCIP_Real transcoefs[3] = {SQRT(2.0), 2.0, 1.0};
+   SCIP_Real transcoefs[3] = {sqrt(2.0), 2.0, 1.0};
    int transcoefsidx[3] = {0, 1, 2};
 
    /* check nlhdlrexprdata*/
@@ -449,7 +452,7 @@ Test(nlhdlrsoc, detectandfree4, .description = "detects simple quadratic express
    /* setup expected data */
    SCIP_EXPR* vars[3] = {xexpr, SCIPexprGetChildren(SCIPexprGetChildren(expr)[2])[0], yexpr};
    SCIP_Real offsets[3] = {0.0, 0.0, 0.0};
-   SCIP_Real transcoefs[3] = {SQRT(2.0), 1.0, 3.0};
+   SCIP_Real transcoefs[3] = {sqrt(2.0), 1.0, 3.0};
    int transcoefsidx[3] = {0, 1, 2};
    int nnonzeroes[3] = {1, 1, 1};
 
@@ -501,7 +504,7 @@ Test(nlhdlrsoc, detectandfree5, .description = "detects more complication quadra
    int nterms = 4;
    SCIP_Real offsets[4] = {0.0, 0.0, 1.0, 0.0};
    int nnonzeroes[4] = {1, 1, 0, 1};
-   SCIP_Real transcoefs[3] = {1.0, SQRT(2.0), 3.0};
+   SCIP_Real transcoefs[3] = {1.0, sqrt(2.0), 3.0};
    int transcoefsidx[3] = {0, 1, 2};
 
    /* check nlhdlrexprdata*/
@@ -588,7 +591,7 @@ Test(nlhdlrsoc, detectandfree7, .description = "detects hyperbolic quadratic exp
    /* setup expected data */
    SCIP_EXPR* vars[4] = {xexpr, zexpr, yexpr, uexpr};
    int nterms = 5;
-   SCIP_Real offsets[5] = {0.0, 0.0, SQRT(2.0), 0.0, 0.0}; /* hyperbolic, constant is second to last of lhs */
+   SCIP_Real offsets[5] = {0.0, 0.0, sqrt(2.0), 0.0, 0.0}; /* hyperbolic, constant is second to last of lhs */
    SCIP_Real transcoefs[6] = {4.0, 2.0, 1.0, -1.0, 1.0, 1.0};
    int transcoefsidx[6] = {0, 1, 2, 3, 2, 3};
    int nnonzeroes[5] = {1, 1, 0, 2, 2};
@@ -679,7 +682,7 @@ Test(nlhdlrsoc, detectandfree9, .description = "detects negated quadratic expres
    int nterms = 4;
    SCIP_Real offsets[4] = {0.0, 0.0, 1.0, 0.0};
    int nnonzeroes[4] = {1, 1, 0, 1};
-   SCIP_Real transcoefs[3] = {1.0, SQRT(2.0), 3.0};
+   SCIP_Real transcoefs[3] = {1.0, sqrt(2.0), 3.0};
    int transcoefsidx[3] = {0, 1, 2};
 
    /* check nlhdlrexprdata*/
@@ -727,7 +730,7 @@ Test(nlhdlrsoc, detectandfree10, .description = "detects negated hyperbolic quad
    /* setup expected data */
    SCIP_EXPR* vars[4] = {xexpr, zexpr, yexpr, uexpr};
    int nterms = 5;
-   SCIP_Real offsets[5] = {0.0, 0.0, SQRT(2.0), 0.0, 0.0};
+   SCIP_Real offsets[5] = {0.0, 0.0, sqrt(2.0), 0.0, 0.0};
    int nnonzeroes[5] = {1, 1, 0, 2, 2};
    SCIP_Real transcoefs[6] = {4.0, 2.0, 1.0, -1.0, 1.0, 1.0};
    int transcoefsidx[6] = {0, 1, 2, 3, 2, 3};
@@ -754,7 +757,7 @@ Test(nlhdlrsoc, detectandfree11, .description = "detects complex quadratic const
    SCIP_Bool infeasible;
    SCIP_Bool success;
 
-   if( !SCIPisIpoptAvailableIpopt() )
+   if( !SCIPlapackIsAvailable() )
       return;
 
    /* create nonlinear constraint */
@@ -781,7 +784,7 @@ Test(nlhdlrsoc, detectandfree11, .description = "detects complex quadratic const
    /* setup expected data */
    SCIP_EXPR* vars[3] = {zexpr, yexpr, xexpr};
    int nterms = 4;
-   SCIP_Real offsets[4] = {-0.5321193159399078, -1.1644355645349063, SQRT(17.0909090909091), 3.275663328435811};
+   SCIP_Real offsets[4] = {-0.5321193159399078, -1.1644355645349063, sqrt(17.0909090909091), 3.275663328435811};
    int nnonzeroes[4] = {3, 3, 0, 3};
    SCIP_Real transcoefs[9] = { 0.8403540860560068, -0.8715100046656585, -0.15866733787070855,
                               -1.5723379860224058, -1.2601695810081268, -1.4058990135251308,
@@ -810,7 +813,7 @@ Test(nlhdlrsoc, detectandfree12, .description = "detects complex quadratic const
    SCIP_Bool infeasible;
    SCIP_Bool success;
 
-   if( !SCIPisIpoptAvailableIpopt() )
+   if( !SCIPlapackIsAvailable() )
       return;
 
    /* create nonlinear constraint */
@@ -838,7 +841,7 @@ Test(nlhdlrsoc, detectandfree12, .description = "detects complex quadratic const
    SCIP_EXPR* vars[3] = {zexpr, yexpr, xexpr};
    int nterms = 4;
    int nnonzeroes[4] = {3, 3, 0, 3};
-   SCIP_Real offsets[4] = {-1.1644355645349063, 0.5321193159399078, SQRT(17.0909090909091), 3.275663328435811};
+   SCIP_Real offsets[4] = {-1.1644355645349063, 0.5321193159399078, sqrt(17.0909090909091), 3.275663328435811};
    SCIP_Real transcoefs[9] = {-1.5723379860224058, -1.2601695810081268, -1.4058990135251308,
                               -0.8403540860560068,  0.8715100046656585,  0.15866733787070855,
                               0.42242364072103755,  0.5895397027601558, -1.0008633075190196};
@@ -857,7 +860,7 @@ Test(nlhdlrsoc, detectandfree12, .description = "detects complex quadratic const
    SCIP_CALL( SCIPreleaseCons(scip, &cons) );
 }
 
-/* detects that SQRT(x^2 -4x + 1) <= 2 is not a soc expression */
+/* detects that sqrt(x^2 -4x + 1) <= 2 is not a soc expression */
 Test(nlhdlrsoc, detectandfree13, .description = "detects complex quadratic constraint")
 {
    SCIP_CONS* cons;
@@ -938,7 +941,7 @@ Test(nlhdlrsoc, detectandfree14, .description = "detects complex quadratic const
 }
 
 
-/* disaggregates SQRT( 8 + 2*(x + 1)^2 + 3*(y + sin(x) + 2)^2 ) <= -2*(w - 1) */
+/* disaggregates sqrt( 8 + 2*(x + 1)^2 + 3*(y + sin(x) + 2)^2 ) <= -2*(w - 1) */
 Test(nlhdlrsoc, disaggregation, .description = "disaggregate soc and check the resulting datastructure")
 {
    SCIP_CONS* cons;
@@ -1006,7 +1009,7 @@ Test(nlhdlrsoc, disaggregation, .description = "disaggregate soc and check the r
 
    SCIP_CALL( SCIPreleaseCons(scip, &cons) );
 
-   /* clear sepastorage to get rid of disaggregation row */
+   /* clear sepastorage to get rid of initial relaxation */
    SCIP_CALL( SCIPclearCuts(scip) );
 }
 
@@ -1018,7 +1021,7 @@ Test(nlhdlrsoc, separation1, .description = "test separation for simple norm exp
    SCIP_EXPR* rootexpr;
    SCIP_EXPR* expr;
    SCIP_SOL* sol;
-   SCIP_ROW* cut;
+   SCIP_ROWPREP* cut;
    SCIP_VAR* cutvars[3];
    SCIP_VAR* auxvar;
    SCIP_Real cutvals[3];
@@ -1044,48 +1047,49 @@ Test(nlhdlrsoc, separation1, .description = "test separation for simple norm exp
    SCIPsetSolVal(scip, sol, nlhdlrexprdata->disvars[1], 1.0);
    SCIPsetSolVal(scip, sol, nlhdlrexprdata->disvars[2], 1.0);
    SCIPsetSolVal(scip, sol, auxvar, 2.0);
+   updateVarVals(scip, nlhdlrexprdata, sol, FALSE);
 
    /* check cut w.r.t. x */
-   SCIP_CALL( generateCutSolDisagg(scip, expr, cons, sol, nlhdlrexprdata, 0, 0.0, 2.0, &cut) );
+   SCIP_CALL( generateCutSolDisagg(scip, &cut, expr, cons, nlhdlrexprdata, 0, 0.0, 2.0) );
 
    cutvars[0] = nlhdlrexprdata->disvars[0];
    cutvars[1] = x;
    cutvars[2] = auxvar;
-   cutvals[0] = -2.0 / SQRT(8.0) - 1.0;
-   cutvals[1] = 4.0 / SQRT(8.0);
-   cutvals[2] = 2.0 / SQRT(8.0) - 1.0;
-   rhs = cutvals[1] + cutvals[2] * 2.0 - SQRT(8.0) + 2.0;
+   cutvals[0] = -2.0 / sqrt(8.0) - 1.0;
+   cutvals[1] = 4.0 / sqrt(8.0);
+   cutvals[2] = 2.0 / sqrt(8.0) - 1.0;
+   rhs = cutvals[1] + cutvals[2] * 2.0 - sqrt(8.0) + 2.0;
 
    checkCut(cut, cutvars, cutvals, rhs, 3);
-   SCIPreleaseRow(scip, &cut);
+   SCIPfreeRowprep(scip, &cut);
 
    /* check cut w.r.t. y */
-   SCIP_CALL( generateCutSolDisagg(scip, expr, cons, sol, nlhdlrexprdata, 1, 0.0, 2.0, &cut) );
+   SCIP_CALL( generateCutSolDisagg(scip, &cut, expr, cons, nlhdlrexprdata, 1, 0.0, 2.0) );
 
    cutvars[0] = auxvar;
    cutvars[1] = y;
    cutvars[2] = nlhdlrexprdata->disvars[1];
-   cutvals[0] = 1.0 / SQRT(17.0) - 1.0;
-   cutvals[1] = 8.0 / SQRT(17.0);
-   cutvals[2] = -1.0 / SQRT(17.0) - 1.0;
-   rhs =  cutvals[0] * 2.0 + cutvals[1] * 2.0 + cutvals[2] * 1.0 - SQRT(17.0) + 3.0;
+   cutvals[0] = 1.0 / sqrt(17.0) - 1.0;
+   cutvals[1] = 8.0 / sqrt(17.0);
+   cutvals[2] = -1.0 / sqrt(17.0) - 1.0;
+   rhs =  cutvals[0] * 2.0 + cutvals[1] * 2.0 + cutvals[2] * 1.0 - sqrt(17.0) + 3.0;
 
    checkCut(cut, cutvars, cutvals, rhs, 3);
-   SCIPreleaseRow(scip, &cut);
+   SCIPfreeRowprep(scip, &cut);
 
    /* check cut w.r.t. z */
-   SCIP_CALL( generateCutSolDisagg(scip, expr, cons, sol, nlhdlrexprdata, 2, 0.0, 2.0, &cut) );
+   SCIP_CALL( generateCutSolDisagg(scip, &cut, expr, cons, nlhdlrexprdata, 2, 0.0, 2.0) );
 
    cutvars[0] = auxvar;
    cutvars[1] = z;
    cutvars[2] = nlhdlrexprdata->disvars[2];
-   cutvals[0] = 1.0 / SQRT(17.0) - 1.0;
-   cutvals[1] = -8.0 / SQRT(17.0);
-   cutvals[2] = -1.0 / SQRT(17.0) - 1.0;
-   rhs =  cutvals[0] * 2.0 - cutvals[1] * 2.0 + cutvals[2] * 1.0 - SQRT(17.0) + 3.0;
+   cutvals[0] = 1.0 / sqrt(17.0) - 1.0;
+   cutvals[1] = -8.0 / sqrt(17.0);
+   cutvals[2] = -1.0 / sqrt(17.0) - 1.0;
+   rhs =  cutvals[0] * 2.0 - cutvals[1] * 2.0 + cutvals[2] * 1.0 - sqrt(17.0) + 3.0;
 
    checkCut(cut, cutvars, cutvals, rhs, 3);
-   SCIPreleaseRow(scip, &cut);
+   SCIPfreeRowprep(scip, &cut);
 
    /* free expr and cons */
    SCIPfreeSol(scip, &sol);
@@ -1101,7 +1105,7 @@ Test(nlhdlrsoc, separation2, .description = "test separation for simple norm exp
    SCIP_EXPR* rootexpr;
    SCIP_EXPR* expr;
    SCIP_SOL* sol;
-   SCIP_ROW* cut;
+   SCIP_ROWPREP* cut;
    SCIP_VAR* cutvars[3];
    SCIP_VAR* auxvar;
    SCIP_Real cutvals[3];
@@ -1142,9 +1146,10 @@ Test(nlhdlrsoc, separation2, .description = "test separation for simple norm exp
    SCIPsetSolVal(scip, sol, x, 1.0);
    SCIPsetSolVal(scip, sol, y, 2.0);
    SCIPsetSolVal(scip, sol, auxvar, 1.0);
+   updateVarVals(scip, nlhdlrexprdata, sol, FALSE);
 
    /* check cut */
-   SCIP_CALL( generateCutSolSOC(scip, expr, cons, sol, nlhdlrexprdata, 0.0, 1.0, &cut) );
+   SCIP_CALL( generateCutSolSOC(scip, &cut, expr, cons, nlhdlrexprdata, 0.0, 1.0) );
 
    cutvars[0] = auxvar;
    cutvars[1] = y;
@@ -1155,7 +1160,7 @@ Test(nlhdlrsoc, separation2, .description = "test separation for simple norm exp
    rhs = -3.0 / 10;
 
    checkCut(cut, cutvars, cutvals, rhs, 3);
-   SCIPreleaseRow(scip, &cut);
+   SCIPfreeRowprep(scip, &cut);
 
    /* free expr and cons */
    SCIPfreeSol(scip, &sol);
@@ -1176,7 +1181,7 @@ Test(nlhdlrsoc, separation3, .description = "test separation for simple expressi
    SCIP_NLHDLREXPRDATA* nlhdlrexprdata = NULL;
    SCIP_EXPR* expr;
    SCIP_SOL* sol;
-   SCIP_ROW* cut;
+   SCIP_ROWPREP* cut;
    SCIP_VAR* cutvars[3];
    SCIP_Real cutvals[3];
    SCIP_Real rhs;
@@ -1201,25 +1206,29 @@ Test(nlhdlrsoc, separation3, .description = "test separation for simple expressi
    getSocNlhdlrData(expr, &nlhdlrexprdata);
    cr_assert_not_null(nlhdlrexprdata);
 
+   /* since nlhdlrInitSepaSoc is not called (on purpose), alloc the varvals array manually */
+   SCIP_CALL( SCIPallocBlockMemoryArray(scip, &nlhdlrexprdata->varvals, nlhdlrexprdata->nvars) );
+
    /* create solution */
    SCIPcreateSol(scip, &sol, NULL);
    SCIPsetSolVal(scip, sol, x, 1.0);
    SCIPsetSolVal(scip, sol, y, 1.0);
    SCIPsetSolVal(scip, sol, z, 0.0);
+   updateVarVals(scip, nlhdlrexprdata, sol, FALSE);
 
    /* check cut */
-   SCIP_CALL( generateCutSolSOC(scip, expr, cons, sol, nlhdlrexprdata, 0.0, 1.0, &cut) );
+   SCIP_CALL( generateCutSolSOC(scip, &cut, expr, cons, nlhdlrexprdata, 0.0, 1.0) );
 
    cutvars[0] = x;
    cutvars[1] = y;
    cutvars[2] = z;
-   cutvals[0] = 4.0 / SQRT( 5.0 );
-   cutvals[1] = -1.0 + 1.0 / SQRT( 5.0 );
-   cutvals[2] = -(5.0 + SQRT( 5.0 )) / 5.0;
+   cutvals[0] = 4.0 / sqrt(5.0);
+   cutvals[1] = -1.0 + 1.0 / sqrt(5.0);
+   cutvals[2] = -(5.0 + sqrt(5.0)) / 5.0;
    rhs = 0.0;
 
    checkCut(cut, cutvars, cutvals, rhs, 3);
-   SCIPreleaseRow(scip, &cut);
+   SCIPfreeRowprep(scip, &cut);
 
    /* free expr and cons */
    SCIPfreeSol(scip, &sol);
@@ -1233,7 +1242,7 @@ Test(nlhdlrsoc, separation3, .description = "test separation for simple expressi
    SCIP_CALL( SCIPreleaseCons(scip, &cons) );
 }
 
-/* detects 2x^2 - 9y^2 + sin(z)^2 < = 0 as soc constraint using a public function */
+/* detects 2x^2 - 9y^2 + z^2 < = 0 as soc constraint using a public function */
 Test(nlhdlrsoc, access, .description = "public detect for simple quadratic constraint")
 {
    SCIP_CONS* cons;
