@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2023 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2024 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -179,6 +179,8 @@ Test(readernl, run, .description = "check running SCIP with -AMPL")
 {
    const char* args[3];
    char solfile[SCIP_MAXSTRLEN];
+   FILE* sol;
+   char scipstatus[100];
 
    /* skip test if nl reader not available (SCIP compiled with AMPL=false) */
    if( SCIPfindReader(scip, "nlreader") == NULL )
@@ -198,15 +200,24 @@ Test(readernl, run, .description = "check running SCIP with -AMPL")
    /* make sure no solfile is there at the moment */
    remove(solfile);
 
+   /* specify some SCIP options via scip_options environment variable (like AMPL scripts would do) */
+   setenv("scip_options", "limits/time 0", 1);
+
    /* run SCIP as if called by AMPL
     * this should writes a sol file (into tests/src/reader, unfortunately)
     */
    SCIP_CALL( SCIPrunShell(3, (char**)args, NULL) );
 
    /* check that a solfile that can be opened exists now */
-   cr_expect_not_null(fopen(solfile, "r"));
+   sol = fopen(solfile, "r");
+   cr_expect_not_null(sol);
+
+   /* check that first line of solfile is the SCIP status, which should be that the timelimit has been reached */
+   cr_expect_not_null(fgets(scipstatus, sizeof(scipstatus), sol));
+   cr_expect_str_eq(scipstatus, "time limit reached\n");
 
    /* cleanup */
+   fclose(sol);
    remove(solfile);
 
    free((char*)args[1]);
@@ -271,3 +282,26 @@ Test(readernl, dualsol, .description = "check whether solving a LP without preso
 
    free(args[1]);
 }
+
+/* check whether initial value of introduced nlobjvar is set */
+Test(readernl, nlobjvar, .description = "check whether initial value of introduced nlobjvar is set")
+{
+   char filename[SCIP_MAXSTRLEN];
+
+   /* skip test if nl reader not available (SCIP compiled with AMPL=false) */
+   if( SCIPfindReader(scip, "nlreader") == NULL )
+      return;
+
+   /* get file to read: suffix1.nl that lives in the same directory as this file */
+   TESTsetTestfilename(filename, __FILE__, "nlobj.nl");
+
+   /* read nl file */
+   SCIP_CALL( SCIPreadProb(scip, filename, NULL) );
+
+   /* we should have an initial solution as well */
+   cr_expect(SCIPgetNSols(scip) == 1);
+
+   /* only if the solution is feasible, it will be available in the transformed problem, too */
+   SCIP_CALL( SCIPtransformProb(scip) );
+   cr_expect(SCIPgetNSols(scip) == 1);
+}
